/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */
package org.wso2.registry.app;

import org.apache.abdera.Abdera;
import org.apache.abdera.factory.Factory;
import org.apache.abdera.model.*;
import org.apache.abdera.parser.Parser;
import org.apache.abdera.protocol.server.Provider;
import org.apache.abdera.protocol.server.RequestContext;
import org.apache.abdera.protocol.server.ResponseContext;
import org.apache.abdera.protocol.server.impl.AbstractProvider;
import org.apache.abdera.protocol.server.impl.AbstractResponseContext;
import org.apache.abdera.protocol.server.impl.BaseResponseContext;
import org.apache.abdera.protocol.server.impl.EmptyResponseContext;
import org.apache.abdera.util.EntityTag;
import org.apache.abdera.util.MimeTypeHelper;
import org.apache.axiom.om.util.Base64;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.Comment;
import org.wso2.registry.*;
import org.wso2.registry.exceptions.ResourceNotFoundException;
import org.wso2.registry.i18n.Messages;
import org.wso2.registry.jdbc.realm.RegistryRealm;
import org.wso2.registry.secure.AuthorizationFailedException;
import org.wso2.registry.secure.SecureRegistry;

import javax.activation.MimeType;
import javax.xml.namespace.QName;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Date;
import java.util.Iterator;
import java.util.Map;

public class AtomRegistry extends AbstractProvider implements Provider, APPConstants {

    private Log log = LogFactory.getLog(AtomRegistry.class);
    public static final String SECURE_REGISTRY = "secureRegistry";

    private Registry jdbcregistry;
    private RegistryRealm realm;

    public AtomRegistry() {
        super(10);
        jdbcregistry = (Registry)System.getProperties().get(RegistryConstants.REGISTRY);
        realm = (RegistryRealm)System.getProperties().get(RegistryConstants.REGISTRY_REALM);
    }

    /**
     * To post an entry we can use  this method , however when it come to jdbcregistry posting means
     * adding new resources , adding comments , adding tags , rating etc..
     * <p/>
     * Depending on the type of the operation URl will change , and the URL will contain operation
     * as "/resource;operation" ,the entry value differ one to another
     *
     * @param request : RequestContext
     * @return EmptyResponseContext(200) if everything has gone well else EmptyResponseContext(500)
     */
    public ResponseContext createEntry(RequestContext request) {
        String entry_id = getRealPath(request);

        String values[] = splitPath(entry_id);
        if (values.length > 1) {
            String modifier = values[1];
            String resourcePath = values[0];
            return processModifierURL(modifier, resourcePath, request);
        }

        // No modifiers, so this is a "straight ahead" creation

        if (!entry_id.endsWith("/")) {
            entry_id += "/";
        }

        String slug = request.getSlug();
        if (slug == null) {
            slug = generateResourceName();
        } else {
            slug = sanitizeResourceName(slug);
        }

        entry_id += slug;

        // Need to add a resource
        Abdera abdera = request.getAbdera();
        ResponseContext rc;

        String contentType = request.getContentType().toString();
        if (IMPORT_MEDIATYPE.equals(contentType)) {
            // This is an import.
            String importURL = request.getParameter("importURL");
            String suggestedPath = request.getSlug();
            Resource metadata = new Resource();
            String location;
            try {
                location = getSecureRegistry(request).importResource(suggestedPath,
                                                                     importURL,
                                                                     metadata);
            } catch (RegistryException e) {
                return new StringResponseContext(abdera, e, 500);
            }
            rc = new EmptyResponseContext(200);
            rc.setLocation(location);
            return rc;
        }

        try {
            Parser parser = abdera.getParser();
            Document<Entry> entry_doc = (Document<Entry>)request.getDocument(parser).clone();

            // Need to add a resource
            Entry entry = entry_doc.getRoot();
//                        if (jdbcregistry.resourceExists(entry_id)) {
//                            try {
//                                resource = jdbcregistry.get(entry_id);
//                            } catch (RegistryException e) {
//                                // Seems like the resource has deleted
//                                resource = new Resource();
//                            }
//                        } else {
            Resource resource = new Resource();
//                        }
            String attValue = entry.getSimpleExtension(new QName(NAMESPACE, "directory"));
            Properties properties = entry.getExtension(PropertyExtensionFactory.PROPERTIES);
            RemoteRegistry.createPropertiesFromExtensionElement(properties, resource);
            String mediaType = entry.getSimpleExtension(new QName(NAMESPACE, "mediaType"));
            if (mediaType != null) {
                resource.setMediaType(mediaType);
            }
            String parentPath = entry.getSimpleExtension(new QName(NAMESPACE, "parentPath"));
            if (parentPath != null && !"".equals(parentPath)) {
                resource.setParentPath(parentPath);
            }
            if (entry.getAuthor().getName() != null) {
                resource.setAuthorUserName(entry.getAuthor().getName());
            }
            if (entry.getSummary() != null) {
                resource.setDescription(entry.getSummary());
            }
            String contentModified = entry.getSimpleExtension(new QName(NAMESPACE,
                                                                        "contentModified"));
            if (attValue != null && "true".equals(attValue)) {
                // Directory
                resource.setDirectory(true);
            } else {
                // Resource
                if (contentModified != null) {
                    String content1 = entry.getContent();
                    if (content1 != null &&
                        !Content.Type.TEXT.equals(entry.getContentType())) {
                        byte[] content = Base64.decode(content1);
                        resource.setContent(content);
                    } else {
                        resource.setContent(content1);
                    }
                }
            }
            //entry-id will tell where to put the resource.
            resource.setPath(entry_id);

            // entry-id will tell where to put the resource.
            String location = getSecureRegistry(request).put(entry_id, resource);

            rc = new EmptyResponseContext(200);
            rc.setLocation(location);
        } catch (Exception e) {
            rc = new StringResponseContext(abdera, e, 500);
        }
        return rc;
    }

    private ResponseContext processModifierURL(String modifier,
                                               String resourcePath,
                                               RequestContext request) {
        Abdera abdera = request.getAbdera();
        try {
            Parser parser = abdera.getParser();
            Document<Entry> entry_doc = (Document<Entry>)request.getDocument(parser).clone();
            Entry entry = entry_doc.getRoot();
            String entryContent = entry.getContent();
            if (PARAMETER_TAGS.equals(modifier)) {
                getSecureRegistry(request).applyTag(resourcePath, entryContent);
            } else if (PARAMETER_COMMENTS.equals(modifier)) {
                Comment comment = new Comment();
                comment.setText(entryContent);
                comment.setUser(entry.getAuthor().getName());
                comment.setTime(entry.getUpdated());
                String commentPath = getSecureRegistry(request).addComment(resourcePath,
                                                                           comment);
                EmptyResponseContext responseContext = new EmptyResponseContext(200);
                responseContext.setLocation(commentPath);
                return responseContext;
            } else if (PARAMETER_RATINGS.equals(modifier)) {
                getSecureRegistry(request).rateResource(resourcePath,
                                                        Integer.parseInt(entryContent));
            } else if (PARAMETER_RENAME.equals(modifier)) {
                getSecureRegistry(request).rename(resourcePath, entryContent);
            } else {
                //TODO : We need to implement the other operation as well
                return new EmptyResponseContext(500);
            }
        } catch (Exception e) {
            return new StringResponseContext(abdera, e, 500);
        }

        return new EmptyResponseContext(200); // For now...
    }

    public ResponseContext deleteEntry(RequestContext request) {
        String entry_id = getRealPath(request);
        if (!"".equals(entry_id)) {
            try {
                getSecureRegistry(request).delete(entry_id);
//                String values[] = splitPath(entry_id);
//                if (values.length > 1) {
//                    String operation = values[1];
////                    String resourcePath = values[0];
//                    if (PARAMETER_TAGS.equals(operation)) {
//                        //TODO Need to implement this
//                    }
//                    //deleting something else, not a resource like Tag or comment
//                } else {
//                    // deleting a resource
//                    getSecureRegistry(request).delete(entry_id);
//                }
                return new EmptyResponseContext(204);
            } catch (RegistryException e) {
                return new StringResponseContext(request.getAbdera(), e, 500);
            }
        } else {
            return new EmptyResponseContext(500);
        }
    }

    /**
     * This method is used for PUT operation and depending on the URL operation will be identify and
     * perform that, not only that depending on the operation the structure of entry may vary
     *
     * @param request : RequestContext
     * @return : EmptyResponseContext(200) if everything has gone well else
     *         EmptyResponseContext(500)
     */
    public ResponseContext updateEntry(RequestContext request) {
        Abdera abdera = request.getAbdera();
        Parser parser = abdera.getParser();
        try {
            MimeType contentType = request.getContentType();
            if (contentType != null && !MimeTypeHelper.isAtom(contentType.toString()))
                return new EmptyResponseContext(415);
            Document<Entry> entry_doc =
                    (Document<Entry>)request.getDocument(parser).clone();
            if (entry_doc != null) {
                Entry entry = entry_doc.getRoot();
                String entry_id = getRealPath(request);
                String values[] = splitPath(entry_id);
                String operation = values[1];
                if (operation != null) {
                    if (operation.indexOf(":") > 0) {
                        operation = operation.substring(0, operation.indexOf(":"));
                    }
                }
                if (PARAMETER_RESTORE.equals(operation)) {
                    String version = values[0];
                    getSecureRegistry(request).restoreVersion(version);
                } else if (PARAMETER_TAGS.equals(operation)) {
                    String path = values[0];
                    String tags = entry.getContent();
                    getSecureRegistry(request).applyTag(path, tags);
                } else if (PARAMETER_TAGS_REMOVED.equals(operation)) {
                    String path = values[0];
                    String tag = entry.getContent();
                    getSecureRegistry(request).removeTag(path, tag);
                } else if (PARAMETER_RATINGS.equals(operation)) {
                    String path = values[0];
                    String rate = entry.getContent();
                    getSecureRegistry(request).rateResource(path, Integer.parseInt(rate));
                } else if (PARAMETER_COMMENTS.equals(operation)) {
                    String commentString = entry.getContent();
                    getSecureRegistry(request)
                            .editComment(values[0] + ";" + values[1], commentString);
                }
            }
            return new EmptyResponseContext(200);
        } catch (Exception pe) {
            return new StringResponseContext(request.getAbdera(), pe, 500);
        }
    }


    /**
     * This method used to get an atom feed representing the jdbcregistry and the depending on the
     * URL the format of the feed will differ
     *
     * @param request : RequestContext
     * @return : Will contains the response feed.
     */
    public ResponseContext getFeed(RequestContext request) {
        Abdera abdera = request.getAbdera();
        String baseURI = request.getBaseUri().toString();
        String entry_id = getRealPath(request);
        if ("/tags".equals(entry_id)) {
            entry_id = "/;tags";
        } else if ("/comments".equals(entry_id)) {
            entry_id = "/;comments";
        }
        int idx = entry_id.indexOf(RegistryConstants.URL_SEPARATOR);
        String resourcePath = entry_id.substring(0, (idx == -1) ? entry_id.length() : idx);

        // See if the resource is there.  If not, no further work is necessary as we'll be
        // returning a 404...
        Resource resource;
        try {
            resource = getSecureRegistry(request).get(resourcePath);
        } catch (ResourceNotFoundException e) {
            return new EmptyResponseContext(404);
        } catch (AuthorizationFailedException e) {
            EmptyResponseContext response = new EmptyResponseContext(401);
            response.setHeader("WWW-Authenticate", "Basic realm=\"WSO2-Registry\"");
            return response;
        } catch (RegistryException e) {
            return new StringResponseContext(request.getAbdera(), e, 500);
        }

        Feed feed;
        if (idx > -1) {
            String operation = entry_id.substring(idx + 1);

            // seems like some we need to do some filtering
            //Checking whether the parameter is there in the operation , like tags:foo
            String parameter = null;
            String parts[] = null;
            if (operation.indexOf(":") > 0) {
                parts = operation.split("\\:");
                parameter = parts[1];
                operation = parts[0];
            }
            if (PARAMETER_VERSION.equals(operation)) {
                // Need to return list of version for the given resource , [resourcePath]
                try {
                    String[] versions = getSecureRegistry(request).getVersions(resourcePath);
                    Factory factory = abdera.getFactory();
                    feed = factory.newFeed();
                    feed.setId("http://wso2.org/jdbcregistry,atom:version");
                    feed.setTitle("Available versions for " + resourcePath);
                    feed.setSubtitle(Messages.getMessage("resource.version", resourcePath));
                    feed.setUpdated(new Date());
                    feed.addLink(request.getBaseUri() + "atom" + resourcePath);
                    feed.addLink(request.getBaseUri() + "atom" + resourcePath, "self");
                    //Versions will be added as entries
                    createEntriesFromStringArray(versions, request.getBaseUri().toString(),
                                                 abdera, "versionLink",
                                                 feed);
                } catch (RegistryException e) {
                    return new StringResponseContext(request.getAbdera(), e, 500);
                }
            } else if (PARAMETER_TAGS.equals(operation)) {
                // if the user [parameter]  name is null then need to return all the
                // tags for the given path
                if (parameter == null) {
                    feed = getFeedForTags(abdera, resourcePath, baseURI, request);
                } else {
                    try {
                        if (parts.length == 2) {
                            feed = getFeedForTagPaths(abdera, parameter, baseURI, request);
                        } else {
                            resource = getSecureRegistry(request).get(entry_id);
                            feed = populateFeed(abdera, baseURI, entry_id, resource, false);
                        }
                    } catch (RegistryException e) {
                        return new StringResponseContext(abdera, e, 500);
                    }
                }

            } else if (PARAMETER_AVERAGE_RATINGS.equals(operation)) {
                feed = getRatingsFeed(abdera, resourcePath, baseURI);
            } else if (PARAMETER_RATINGS.equals(operation)) {
                try {
                    resource = getSecureRegistry(request).get(entry_id);
                    feed = populateFeed(abdera, baseURI, entry_id, resource, false);
                    if (parameter == null) {
                        feed.addSimpleExtension(QNAME_AVGRATING,
                                                "" + jdbcregistry.getAverageRating(resourcePath));
                    }
                } catch (ResourceNotFoundException e) {
                    return new EmptyResponseContext(404);
                } catch (RegistryException e) {
                    return new StringResponseContext(abdera, e, 500);
                }
            } else if (PARAMETER_COMMENTS.equals(operation)) {
                if (parameter == null) {
                    feed = getFeedForComments(abdera, entry_id, request);
                } else {
                    try {
                        resource = getSecureRegistry(request).get(entry_id);
                    } catch (RegistryException e) {
                        return new StringResponseContext(abdera, e, 500);
                    }
                    feed = populateFeed(abdera, baseURI, entry_id, resource, false);
                }
            } else if (PARAMETER_TAG_PATHS.equals(operation)) {
                feed = getFeedForTagPaths(abdera, parameter, baseURI, request);
            } else if (PARAMETER_LOGS.equals(operation)) {
                Date to = request.getDateHeader("ToDate");
                Date from = request.getDateHeader("FromDate");
                String action = request.getHeader("Action");
                String author = request.getHeader("Author");
                try {
                    feed = getFeedForLogs(abdera, resourcePath, from, to, author, action, request);
                } catch (RegistryException e) {
                    return new StringResponseContext(request.getAbdera(), e, 500);
                }
            } else if (PARAMETER_QUERY.equals(operation)) {
                try {
                    // Trying to execute a query for a given path for a given paramater
                    String parms = request.getHeader("parameters");
                    Map parameters = RemoteRegistry.decodeParametesAsString(parms);
                    resource = getSecureRegistry(request).executeQuery(resourcePath, parameters);
                    feed = populateFeed(abdera, baseURI, resourcePath, resource, true);
                } catch (RegistryException e) {
                    return new StringResponseContext(request.getAbdera(), e, 500);
                }
            } else if (PARAMETER_RESOURCE_EXIST.equals(operation)) {
                feed = getFeedResourceExist(abdera, resourcePath, request);
            } else {
                return createErrorResponse();
            }
        } else {
            //need to return a feed representing resource.
            feed = populateFeed(abdera, baseURI, entry_id, resource, false);
        }

        if (feed != null) return getResponseContextForFeed(feed);

        return createErrorResponse();
    }

    private AbstractResponseContext getResponseContextForFeed(Feed feed) {
        AbstractResponseContext rc;
        Document<Feed> docfeed = feed.getDocument();
        rc = new BaseResponseContext<Document<Feed>>(docfeed);
        rc.setEntityTag(calculateEntityTag(docfeed.getRoot()));
        return rc;
    }

    /**
     * This method will generate a Error response for resource not found
     *
     * @return : Created ResponseContext
     */
    private ResponseContext createErrorResponse() {
        EmptyResponseContext responseContext = new EmptyResponseContext(500);
        responseContext.setStatusText(Messages.getMessage("resource.not.found"));
        return responseContext;
    }

    /**
     * This method will serialized Resource for the given path into a Atom feed. If the resource is
     * a directory, then there will be multiple entry inside that representing the child nodes. Here
     * there will be special simpleExtension to represent called "directory" and the value would be
     * "true"
     * <p/>
     * If the return resource is a resource which contains comment then the same thing will happen
     * <p/>
     * if the resource is actual resource then there will be one entry inside the feed and that
     * entry will have the content of the resource as byte array and when serialize , which will
     * become base64 , if the content is String then that will appear as it is.
     *
     * @param abdera       : Reference to Abdera object
     * @param baseURI      : BaseURI of the jdbcregistry
     * @param path         : resource path
     * @param resource     : Resource object that we need to generate feed
     * @param exceuteQuery : To indicate whether the method call is for query execute result or
     *                     something else
     * @return Feed : which represent the atom feed of the resource.
     */
    private Feed populateFeed(Abdera abdera,
                              String baseURI,
                              String path,
                              Resource resource,
                              boolean exceuteQuery) {
        Feed feed = getFeedInstance(abdera);
        //Properties in the resource will be send as attributes in the feed object
        RemoteRegistry.addPropertyExtensionElement(resource.getProperties(),
                                                   abdera,
                                                   feed,
                                                   PropertyExtensionFactory.PROPERTIES,
                                                   PropertyExtensionFactory.PROPERTY);
        //if the resource is deleted then there will be comment saying that the resource is
        // deleted
        if (resource.getState() == RegistryConstants.DELETED_STATE) {
            feed.addSimpleExtension(new QName(NAMESPACE, "state"), "Deleted");
        }
        feed.setId("http://wso2.org/jdbcregistry:atom" + resource.getId());
        if (resource.getDescription() != null && !"".equals(resource.getDescription())) {
            feed.setSubtitle(resource.getDescription());
        }
        if (resource.getAuthorUserName() != null) {
            feed.addAuthor(resource.getAuthorUserName());
        }
        feed.setTitle(path);
        if (resource.getLastModified() != null) {
            feed.setUpdated(resource.getLastModified());
        } else {
            feed.setUpdated(new Date());
        }
//        resource.getCreatedTime()
        if (resource.getCreatedTime() != null) {
            feed.addSimpleExtension(new QName(NAMESPACE, "createdTime"),
                                    "" + resource.getCreatedTime().getTime());
        }
        feed.addSimpleExtension(new QName(NAMESPACE, "mediaType"), resource.getMediaType());
        feed.addSimpleExtension(new QName(NAMESPACE, "parentPath"), resource.getParentPath());
        path = RemoteRegistry.encodeURL(path);
        String href = baseURI + "atom" + path;
        feed.addLink(href);
        feed.addLink(href, "self");
        feed.addSimpleExtension(new QName(NAMESPACE, "lastUpdatedUser"),
                                resource.getLastUpdaterUserName());
        //if the content type is comment then the resource will contains ,
        // String array of comments
        if (resource.isDirectory()) {
            feed.addSimpleExtension(new QName(NAMESPACE, "directory"), "true");
        }
        if (exceuteQuery) {
            String values[] = (String[])resource.getContent();
            for (String value : values) {
                Entry entry = abdera.getFactory().newEntry();
                //adding child node as and setting the ref as path
                entry.addLink(value, "queryResult");
                feed.addEntry(entry);
            }
            return feed;
        } else {
            if ("comments".equals(resource.getMediaType())) {
                String rel = "commentLink";
                createEntriesFromStringArray((String[])resource.getContent(),
                                             baseURI, abdera, rel, feed);
                return feed;
            }
            if (resource.isDirectory()) {
                String rel = "path";
                createEntriesFromStringArray((String[])resource.getContent(),
                                             baseURI, abdera, rel, feed);
                return feed;
            } else {
                //Now the resource is a actual resource , so need to send the content
                //adding attribute to indicate the resource is not a directory
                feed.addSimpleExtension(new QName(NAMESPACE, "directory"), "false");
                Object content = resource.getContent();
                Entry entry = abdera.getFactory().newEntry();
                String link = baseURI + "atom" + path;
                entry.setId(link);
                entry.setTitle(path);

                entry.setUpdated(feed.getUpdated());
                entry.setPublished(resource.getCreatedTime());
                String resourcePath = baseURI + "resources" + path;
                if (content != null && content instanceof byte[]) {
                    entry.addLink(resourcePath, "edit-media");
                    feed.addLink(resourcePath, "self");
                } else {
                    if (content != null) {
                        entry.setContent(content.toString());
                    }
                }
                if (resource.getAuthorUserName() != null) {
                    entry.addAuthor(resource.getAuthorUserName());
                }
                entry.addLink(resourcePath, "alternate");
                feed.addEntry(entry);
            }
        }
        return feed;
    }

    /**
     * This will create incomplete entry for a given String array
     *
     * @param values  : String array
     * @param baseURI : Base URI of the jdbcregistry
     * @param abdera  : Instance of Abdera object
     * @param rel     : The rel value
     * @param feed    : Feed object where we need to add the created entry
     */
    private void createEntriesFromStringArray(String[] values,
                                              String baseURI,
                                              Abdera abdera,
                                              String rel,
                                              Feed feed) {
        String authorName = null;
        Person person = feed.getAuthor();
        if (person != null) {
            authorName = person.getName();
            feed.addAuthor(authorName);
        }
        for (String value : values) {
            String nodeLink = baseURI + "atom" + value;
            nodeLink = RemoteRegistry.encodeURL(nodeLink);
            Entry entry = abdera.getFactory().newEntry();
            entry.addLink(nodeLink);
            entry.setTitle(value);
            entry.setUpdated(new Date());
            value = RemoteRegistry.encodeURL(value);
            entry.addLink(nodeLink, "self");
            //adding child node as and setting the ref as path
            entry.addLink(value, rel);
            if (authorName != null) {
                entry.addAuthor(authorName);
            }
            entry.setId(nodeLink);
            entry.setContent("");
            feed.addEntry(entry);
        }
    }


    public ResponseContext getEntry(RequestContext request) {
        Abdera abdera = request.getAbdera();
        String baseURI = request.getBaseUri().toString();
        String entry_id = getRealPath(request);
        Entry entry = null;
        if ("".equals(entry_id)) {
            try {
                Resource resource =
                        getSecureRegistry(request).get(RegistryConstants.PATH_SEPARATOR);
                if (resource == null) {
                    entry = getEmptyEntry(abdera, baseURI, get_feed_doc(abdera, baseURI));
                }
            } catch (RegistryException e) {
                log.error(e);
                return new EmptyResponseContext(404);
            }
        } else {

        }
        if (entry != null) {
            Feed feed = entry.getParentElement();
            entry.setSource(feed.getAsSource());
            Document<Entry> entry_doc = entry.getDocument();
            AbstractResponseContext rc = new BaseResponseContext<Document<Entry>>(entry_doc);
            rc.setEntityTag(calculateEntityTag(entry));
            return rc;
        } else {
            return new EmptyResponseContext(404);
        }
    }

    private Document<Feed> get_feed_doc(Abdera abdera, String baseUri) {
        Feed feed = getFeedInstance(abdera);
        try {
            feed.setId("http://wso2.org/jdbcregistry,2007:feed");
            feed.setTitle("WSO2 Registry");
            feed.setUpdated(new Date());
            feed.addLink(baseUri);
            feed.addLink("", "self");
            return feed.getDocument();
        } catch (Exception e) {
            log.error(e);
        }
        return null;
    }

    private Entry getEmptyEntry(Abdera abdera, String baseURI, Document<Feed> feed) {
        Entry entry = abdera.getFactory().newEntry();
        entry.setId(baseURI + "atom/");
        entry.setTitle("Registry root");
        entry.setUpdated(new Date());
        entry.setPublished(new Date());
        entry.addLink(baseURI + "atom/");
        feed.getRoot().addEntry(entry);
        return entry;
    }

    /**
     * This method will return the path by removing the atom and the base URL from it , For example
     * if the path is "localhost:8080/wso2registry/atom/r2" , then the method will return "/r2"
     *
     * @param request : RequestContext
     * @return real path removing unwanted part from that.
     */
    private String getRealPath(RequestContext request) {
        String path = request.getBaseUri().getPath();
        String uri = request.getUri().toString();
        uri = uri.replaceAll(path + "atom", "");
        try {
            uri = URLDecoder.decode(uri, "utf-8");
        } catch (UnsupportedEncodingException e) {
            // TODO : do anything here?
        }
        return (uri.length() == 0) ? "/" : uri;
    }

    /**
     * This method will split the path into parts around the path separator, returning an array of
     * the parts.
     *
     * @param path : URL path
     * @return String array of the splited string
     */
    private String[] splitPath(String path) {
        return path.split("\\" + RegistryConstants.URL_SEPARATOR);
    }

    private EntityTag calculateEntityTag(Base base) {
        String id = null;
        String modified = null;
        if (base instanceof Entry) {
            id = ((Entry)base).getId().toString();
            modified = ((Entry)base).getUpdatedElement().getText();
        } else if (base instanceof Feed) {
            id = ((Feed)base).getId().toString();
            modified = ((Feed)base).getUpdatedElement().getText();
        }
        return EntityTag.generate(id, modified);
    }

    /**
     * This will generate Feed element representing all the tags for the given path Title of the
     * entry will be the tag name and the there will be an extension element called "taggings" to
     * represent the tag count
     *
     * @param abdera  : Instance of Abdera object
     * @param path    : Path of which need to get the tags
     * @param baseUri : Base URI of the jdbcregistry
     * @param request : RequestContext
     * @return : Feed object which contains the tags
     */
    private Feed getFeedForTags(Abdera abdera,
                                String path,
                                String baseUri,
                                RequestContext request) {
        try {
            Feed feed = getFeedInstance(abdera);
            Tag tags[] = getSecureRegistry(request).getTags(path);
            feed.setId("http://wso2.org/jdbcregistry:tags" + path);
            feed.setTitle("Tags for " + path);
            String nodeLink = baseUri + "atom" + path +
                              RegistryConstants.URL_SEPARATOR +
                              PARAMETER_TAGS;
            feed.addLink(nodeLink);
            feed.setUpdated(new Date());

            for (Tag tag : tags) {
                Entry entry = abdera.getFactory().newEntry();
                entry.setTitle(tag.getTagName());
                entry.setContent(tag.getTagName());
                entry.addSimpleExtension(new QName(NAMESPACE, "taggings"), "" + tag.getTagCount());
                feed.addEntry(entry);
            }
            return feed;
        } catch (RegistryException e) {
            return null;
        }
    }

    /**
     * This method will give a feed object instance
     *
     * @param abdera : Instance to Abdera
     * @return created Feed object
     */
    private Feed getFeedInstance(Abdera abdera) {
        Factory factory = abdera.getFactory();
        return factory.newFeed();
    }

    /**
     * Get an Atom feed for the ratings of a given resource.  This feed will contain an extension
     * "AverageRating", and entries for each individual rating.
     *
     * @param abdera  : Abdera instance
     * @param path    : Resource path
     * @param baseUri : Base URI of the jdbcregistry
     * @return Feed representing the average feed
     */
    private Feed getRatingsFeed(Abdera abdera, String path, String baseUri) {
        try {
            Feed feed = getFeedInstance(abdera);
            feed.setUpdated(new Date());
            feed.setId("http://wso2.org/jdbcregistry:averageRating");
            feed.setTitle("Average Rating for the resource " + path);
            String nodeLink = baseUri + "atom" + path +
                              RegistryConstants.URL_SEPARATOR +
                              PARAMETER_AVERAGE_RATINGS;
            feed.addLink(nodeLink);
            feed.addSimpleExtension(QNAME_AVGRATING,
                                    "" + jdbcregistry.getAverageRating(path));
            return feed;
        } catch (RegistryException e) {
            return null;
        }
    }

    /**
     * To get the collection of comments for a given path a feed element will be generated with a
     * set of entry . And each entry will represent - commented Author - Commented Time - And the
     * content
     *
     * @param abdera  : An instance of Abdera
     * @param path    : Resource path
     * @param request : RequestContext
     * @return : created Feed object
     */
    private Feed getFeedForComments(Abdera abdera, String path, RequestContext request) {
        try {
            Feed feed = getFeedInstance(abdera);
            feed.addSimpleExtension(new QName(NAMESPACE, "directory"), "true");
            feed.addSimpleExtension(COMMENTS_QNAME, "true");
            Comment comments[] = getSecureRegistry(request).getComments(path);
            feed.setId("http://wso2.org/registry,Comments" + path);
            feed.setUpdated(new Date());
            feed.setTitle("comments for the resource " + path);
            for (Comment comment : comments) {
                Entry entry = abdera.getFactory().newEntry();
                entry.addLink(comment.getResourcePath(), "path");
                entry.addLink(comment.getCommentPath(), "direct");
                entry.addLink(comment.getCommentPath(), "self");
                entry.setTitle(comment.getResourcePath());
                entry.setId("http://wso2.org/registry" + comment.getCommentPath());
                entry.addAuthor(comment.getUser());
                entry.setUpdated(comment.getTime());
                entry.setContent(comment.getText());
                feed.addEntry(entry);
            }
            return feed;
        } catch (RegistryException e) {
            return null;
        }
    }

    /**
     * To get the the resource paths for a given tag , we can use this method. And the method will
     * generate Feed containing entries , where each represent one resource path.
     * <p/>
     * Link with the path will represent the path and the and the tag count will be represent using
     * an extension element called "taggings"
     *
     * @param abdera  : An instance of Abdera
     * @param tag     : Nam of the tag
     * @param baseUri : Base URI of the jdbcregistry
     * @param request : RequestContext
     * @return : Feed containing one or more entries
     */
    private Feed getFeedForTagPaths(Abdera abdera,
                                    String tag,
                                    String baseUri,
                                    RequestContext request) {
        try {
            Feed feed = getFeedInstance(abdera);
            TaggedResourcePath tagpaths[] = getSecureRegistry(request).getResourcePathsWithTag(tag);
            feed.setId("http://wso2.org/jdbcregistry/TagPaths");
            feed.setUpdated(new Date());
            feed.setTitle("Resource path for " + tag);
            for (TaggedResourcePath tagPath : tagpaths) {
                Entry entry = abdera.getFactory().newEntry();
                String path = tagPath.getResourcePath();
                entry.setTitle(path);
                entry.addSimpleExtension(new QName(NAMESPACE, "taggings"),
                                         "" + tagPath.getTagCount());
                Map tagCounts = tagPath.getTagCounts();
                Iterator iCounts = tagCounts.keySet().iterator();
                java.util.Properties properties = new java.util.Properties();
                while (iCounts.hasNext()) {
                    String key = (String)iCounts.next();
                    String count = (tagCounts.get(key)).toString();
                    properties.put(key, count);
                }
                RemoteRegistry.addPropertyExtensionElement(properties,
                                                           abdera,
                                                           entry,
                                                           PropertyExtensionFactory.TAGS,
                                                           PropertyExtensionFactory.TAG);
//                entry.addSimpleExtension(new QName("tagCounts"), "" + tagPath.getTagCount())
                entry.addLink(baseUri + "atom" + path, "path");
                entry.addLink(baseUri + "atom" + path);
                feed.addEntry(entry);
            }
            return feed;
        } catch (RegistryException e) {
            return null;
        }
    }

    public ResponseContext getService(RequestContext request) {
        Abdera abdera = request.getAbdera();
        EntityTag service_etag = new EntityTag("jdbcregistry");
        Document<Service> service;
        Factory factory = abdera.getFactory();
        Service serviceElement = factory.newService();
        Workspace workspace = serviceElement.addWorkspace("Registry");
        try {
            Collection collection = workspace.addCollection("jdbcregistry", "atom/feed");
            collection.setAccept("entry");
            collection.addCategories().setFixed(false);
        } catch (Exception e) {
            log.error(e);
        }
        service = serviceElement.getDocument();
        AbstractResponseContext rc;
        rc = new BaseResponseContext<Document<Service>>(service);
        rc.setEntityTag(service_etag);
        return rc;
    }

    /**
     * Will create a feed element for user asked logs , there will be separate entry for log and
     * each entry will contain - edited date : Log date - content : log action data - Author : Log
     * author - Extension element for action - Extension element for path
     *
     * @param abdera  : An instance of an Abdera
     * @param path    : Resource path
     * @param from    : From date , will be a Date Header in the request
     * @param to      : To data , will be a Date Header in the request
     * @param author  : Author , will be a string header in the request
     * @param action  : Log action , will be a string header in the request
     * @param request : RequestContext
     * @return : Feed containing results
     * @throws RegistryException : If something went wrong
     */
    private Feed getFeedForLogs(Abdera abdera,
                                String path,
                                Date from,
                                Date to,
                                String author,
                                String action,
                                RequestContext request) throws RegistryException {
        LogEntry[] entries = getSecureRegistry(request).getLogs(
                path, Integer.parseInt(action), author, from, to, true);
        Feed feed = getFeedInstance(abdera);
        feed.setId("http://wso2.org/jdbcregistry,2007:logs");
        feed.setTitle("Logs for the resource" + path);
        feed.addLink("", "self");
        feed.setUpdated(new Date());
        for (LogEntry logentry : entries) {
            Entry entry = abdera.getFactory().newEntry();
            entry.setEdited(logentry.getDate());
            entry.setContent(logentry.getActionData());
            entry.addAuthor(logentry.getUserName());
            entry.addSimpleExtension(new QName(NAMESPACE, "action"), "" + logentry.getAction());
            entry.addSimpleExtension(new QName(NAMESPACE, "path"), logentry.getResourcePath());
            feed.addEntry(entry);
        }
        return feed;
    }

    /**
     * This method to get a feed element representing , whether a feed exist or not
     *
     * @param abdera  : An instance  of Abdera
     * @param path    : Resource path
     * @param request : RequestContext
     * @return : Feed object
     */
    private Feed getFeedResourceExist(Abdera abdera,
                                      String path,
                                      RequestContext request) {
        try {
            boolean isResourceExsit = getSecureRegistry(request).resourceExists(path);
            Feed feed = getFeedInstance(abdera);
            feed.setUpdated(new Date());
            feed.setId("http://wso2.org/jdbcregistry:resourceExist" + path);
            if (isResourceExsit) {
                feed.setSubtitle("True");
            } else {
                feed.setSubtitle("False");
            }
            return feed;
        } catch (RegistryException e) {
            return null;
        }
    }

    /**
     * This method will create a SecureRegistry for APP. If the user has send the user name and the
     * password then the jdbcregistry will be created using the value he has given, else the
     * jdbcregistry instance will be created as annon user.
     *
     * @param request : RequestContext to get the authorization header
     * @return : Created jdbcregistry
     * @throws RegistryException : If something went wrong
     */
    private Registry getSecureRegistry(RequestContext request) throws RegistryException {
        String authorizationString = request.getAuthorization();
        if (authorizationString != null) {
            // spliting the Authorization string "Basic QWxhZGRpbjpvcGVuIHNlc2FtZQ=="
            String values[] = authorizationString.split("\\ ");
            if ("Basic".equals(values[0])) {
                //decoding the username and passord
                authorizationString = new String(Base64.decode(values[1]));
                //spliting the decode sting to get the userName and the password;
                values = authorizationString.split("\\:");
                String userName = values[0];
                String password = values[1];
                // user name and password have found
                return new SecureRegistry(
                        userName, password, jdbcregistry, realm);
            } else {
                throw new RegistryException("Invalid Authorization string");
            }
        } else {
            // seems like need to create a secure jdbcregistry with annon user
            return new SecureRegistry(
                    RegistryConstants.ANONYMOUS_USER, "guest", jdbcregistry, realm);
        }
    }

    int curResource = 1;

    private synchronized String generateResourceName() {
        return "resource" + curResource++;
    }

    private String sanitizeResourceName(String name) {
        return name.replaceAll("[\\?\\&\\(\\)]", "");
    }
}
