/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */
package org.wso2.registry.app;

import org.apache.abdera.Abdera;
import org.apache.abdera.factory.Factory;
import org.apache.abdera.model.*;
import org.apache.abdera.protocol.Response;
import org.apache.abdera.protocol.client.AbderaClient;
import org.apache.abdera.protocol.client.ClientResponse;
import org.apache.abdera.protocol.client.RequestOptions;
import org.apache.axiom.om.util.Base64;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.Comment;
import org.wso2.registry.*;
import org.wso2.registry.exceptions.ResourceNotFoundException;
import org.wso2.registry.i18n.Messages;

import javax.xml.namespace.QName;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.*;

public class RemoteRegistry implements Registry, APPConstants {

    private String baseURI;
    private Log log = LogFactory.getLog(RemoteRegistry.class);
    //This will keep the value of username and the password for authorization
    private String authorizationString = null;
    private String username = null;

    /**
     * To create a remote registry need to provide a URL of a remote registry and the URL should be
     * something like http://localhost:8080/wso2registry/atom/r1
     *
     * @param registryURL : URL to the registry or to the resource
     */
    public RemoteRegistry(URL registryURL) {
        baseURI = registryURL.toString();
        if (baseURI.endsWith("/")) {
            baseURI = baseURI.substring(0, baseURI.length() - 1);
        }
    }

    /**
     * To create a remote registry to connect to a secure registry or a registry where we need to
     * pass userName and the password to perform any operation. Passing not null value registry will
     * send the username and the password for each request. Need to remember here is that the
     * username and the password send as plain text.
     *
     * @param registryURL : URL to the registry or to the resource
     * @param userName    : user name of the registry , value can be null , if the value is null
     *                    then the user will be the anonymous.
     * @param password    : password , value can be null
     */
    public RemoteRegistry(URL registryURL, String userName, String password) {
        baseURI = registryURL.toString();
        if (baseURI.endsWith("/")) {
            baseURI = baseURI.substring(0, baseURI.length() - 1);
        }
        this.username = userName;
        if (userName != null && password != null) {
            authorizationString = userName + ":" + password;
            authorizationString = "Basic " + Base64.encode(authorizationString.getBytes());
        }
    }

    public Resource get(String path) throws RegistryException {
        Abdera abdera = new Abdera();
        AbderaClient abderaClient = new AbderaClient(abdera);
        ClientResponse clientResponse =
                abderaClient.get(baseURI + encodeURL(path), getAuthorization());
        if (clientResponse.getType() == Response.ResponseType.CLIENT_ERROR ||
            clientResponse.getType() == Response.ResponseType.SERVER_ERROR) {
            throw new ResourceNotFoundException(path);
        }
        Document introspection = clientResponse.getDocument();
        Feed feed = (Feed)introspection.getRoot();
        String state = feed.getSimpleExtension(new QName(NAMESPACE, "state"));
        if (state != null && state.equals("Deleted")) {
            throw new ResourceNotFoundException(path);
        }
        return createResourceFromFeed(feed, false);
    }

    /**
     * This method will generate a resource object representing the Feed object and the logic will
     * be simply the reverse of the create feed of the Atom registry
     *
     * @param feed    : Feed object which represent a resource object
     * @param isQuery : To tell whether we are going to execute a query of trying to just get a
     *                resource
     * @return : Created resource
     * @throws org.wso2.registry.RegistryException
     *          : If user is unable to open the URL connection
     */
    private Resource createResourceFromFeed(Feed feed, boolean isQuery) throws RegistryException {
        String isDirectory = feed.getSimpleExtension(new QName(NAMESPACE, "directory"));
        Resource resource = new Resource();
        Properties properties = feed.getExtension(PropertyExtensionFactory.PROPERTIES);
        RemoteRegistry.createPropertiesFromExtensionElement(properties, resource);
        if (feed.getAuthor() != null) {
            resource.setAuthorUserName(feed.getAuthor().getName());
        }
        resource.setLastModified(feed.getUpdated());
        String createdDate = feed.getSimpleExtension(new QName(NAMESPACE, "createdTime"));
        if (createdDate != null) {
            resource.setCreatedTime(new Date(Long.parseLong(createdDate)));
        }

        String lastUpdatedUser = feed.getSimpleExtension(new QName(NAMESPACE,
                                                                   "lastUpdatedUser"));
        if (lastUpdatedUser != null) {
            resource.setLastUpdaterUserName(lastUpdatedUser);
        }
        String mediaType = feed.getSimpleExtension(new QName(NAMESPACE, "mediaType"));
        if (mediaType != null) {
            resource.setMediaType(mediaType);
        }
        String parentPath = feed.getSimpleExtension(new QName(NAMESPACE, "parentPath"));
        if (parentPath != null) {
            resource.setParentPath(parentPath);
        }
        resource.setPath(feed.getTitle());
        resource.setDescription(feed.getSubtitle());
        String state = feed.getSimpleExtension(new QName(NAMESPACE, "state"));
        if (state != null && "Deleted".equals(state)) {
            resource.setState(RegistryConstants.DELETED_STATE);
        }

        if (isDirectory != null && "true".equals(isDirectory)) {
            resource.setDirectory(true);
            String isComments = feed.getSimpleExtension(COMMENTS_QNAME);
            if (isComments != null) {
                resource.setContent(getCommentsFromFeed(feed));
            } else {
                List entries = feed.getEntries();
                if (entries != null) {
                    String[] childNodes = new String[entries.size()];
                    for (int i = 0; i < entries.size(); i++) {
                        Entry entry = (Entry)entries.get(i);
                        if (isQuery) {
                            childNodes[i] = entry.getLink("queryResult").getHref().toString();
                        } else {
                            childNodes[i] = entry.getLink("path").getHref().toString();
                        }

                    }
                    resource.setContent(childNodes);
                }
            }
        } else {
            List entries = feed.getEntries();
            // if the resource is not a directory then we have only one entry
            // if the resource is not a directory then we have only one entry
            Entry entry = (Entry)entries.get(0);
            Link link = entry.getLink("edit-media");
            if (link != null) {
                try {
                    URL url = new URL(link.getHref().toString());
                    URLConnection uc = url.openConnection();
                    if (authorizationString != null) {
                        uc.setRequestProperty("Authorization", authorizationString);
                    }
                    InputStream in = url.openStream();
                    int read;
                    ByteArrayOutputStream out = new ByteArrayOutputStream();
                    while ((read = in.read()) > 0) {
                        out.write(read);
                    }
                    out.flush();
                    resource.setContent(out.toByteArray());
                    return resource;
                } catch (IOException e) {
                    throw new RegistryException(Messages.getMessage("unable.to.receive",
                                                                    link.getHref().toString()));
                }
            }
            String content1 = entry.getContent();
            resource.setContent(content1);
        }
        return resource;
    }

    public boolean resourceExists(String path) throws RegistryException {
        Abdera abdera = new Abdera();
        AbderaClient abderaClient = new AbderaClient(abdera);
        ClientResponse response = abderaClient.head(baseURI + encodeURL(path), getAuthorization());
        return (response.getType() == Response.ResponseType.SUCCESS);
    }

    public String put(String suggestedPath, Resource resource) throws RegistryException {
        // Figure out where we're POSTing to...

        int idx = suggestedPath.lastIndexOf("/");
        String relativePath = suggestedPath.substring(idx + 1);
        String parentPath = idx > 1 ? suggestedPath.substring(0, idx) : "/";

        Abdera abdera = new Abdera();
        AbderaClient abderaClient = new AbderaClient(abdera);
        Entry entry = abdera.getFactory().newEntry();
        entry.setSummary(resource.getDescription());
        if (resource.getAuthorUserName() != null) {
            entry.addAuthor(resource.getAuthorUserName());
        } else {
            entry.addAuthor(username);
        }
        java.util.Properties properties = resource.getProperties();
        addPropertyExtensionElement(properties, abdera, entry, PropertyExtensionFactory.PROPERTIES,
                                    PropertyExtensionFactory.PROPERTY);
        entry.addSimpleExtension(new QName(NAMESPACE, "mediaType"), resource.getMediaType());
        entry.addSimpleExtension(new QName(NAMESPACE, "parentPath"), resource.getParentPath());
        if (resource.isContentModified()) {
            entry.addSimpleExtension(new QName(NAMESPACE, "contentModified"), "true");
        }
        if (resource.isDirectory()) {
            entry.addSimpleExtension(new QName(NAMESPACE, "directory"), "true");
        } else {
            entry.addSimpleExtension(new QName(NAMESPACE, "directory"), "false");
            Object content = resource.getContent();
            if (content != null && content instanceof byte[]) {
                ByteArrayInputStream in = new ByteArrayInputStream((byte[])content);
                entry.setContent(in);
            } else if (content instanceof InputStream) {
                entry.setContent((InputStream)content);
            } else {
                entry.setContent((String)content);
            }
        }

        RequestOptions requestOptions = getAuthorization();
        requestOptions.setSlug(relativePath);

        ClientResponse resp =
                abderaClient.post(baseURI + encodeURL(parentPath), entry, requestOptions);
        if (resp.getType() == Response.ResponseType.SUCCESS) {
            log.info(Messages.getMessage("resource.add", suggestedPath));
        } else {
            log.error(Messages.getMessage("add.resource.fail", suggestedPath));
            throw new RegistryException(Messages.getMessage("add.resource.fail", suggestedPath));
        }
        return suggestedPath;
    }


    public static void createPropertiesFromExtensionElement(Properties properties,
                                                            Resource resource) {
        if (properties != null) {
            List propertyList = properties.getExtensions(PropertyExtensionFactory.PROPERTY);
            for (Object aPropertyList : propertyList) {
                Property property = (Property)aPropertyList;
                PropertyName pn = property.getExtension(PropertyExtensionFactory.PROPERTY_NAME);
                PropertyValue pv = property.getExtension(PropertyExtensionFactory.PROPERTY_VALUE);
                resource.setProperty(pn.getText(), pv.getText());
            }
        }
    }

    /**
     * This will generate extension element and that will add to the entry , the created element
     * will be something like <node1name> <node2Name> + <name></name> <value></value> </node2Name>
     * </node1name>
     *
     * @param properties : List of Name value pairs
     * @param abdera     : Abdera instance
     * @param entry      : Instanc of entery where extension element need to add
     * @param node1name  : Name of node 1
     * @param node2Name  : Name of node 2
     */
    public static void addPropertyExtensionElement(java.util.Properties properties,
                                                   Abdera abdera,
                                                   ExtensibleElement entry,
                                                   QName node1name,
                                                   QName node2Name) {
        if (properties != null) {
            Factory factory = abdera.getFactory();
            Properties propertyElement = factory.newExtensionElement(node1name);
            Iterator keys = properties.keySet().iterator();
            while (keys.hasNext()) {
                String key = (String)keys.next();
                Property property = factory.newExtensionElement(node2Name);
                PropertyName pn = factory.newExtensionElement(
                        PropertyExtensionFactory.PROPERTY_NAME);
                pn.setPropertyName(key);
                PropertyValue pv = factory.newExtensionElement(
                        PropertyExtensionFactory.PROPERTY_VALUE);
                pv.setPropertyValue((String)properties.get(key));
                property.setProperty(pn, pv);
                propertyElement.setProperty(property);
            }
            entry.addExtension(propertyElement);
        }
    }


    public String importResource(String suggestedPath, String sourceURL, Resource metadata)
            throws RegistryException {
        Abdera abdera = new Abdera();
        AbderaClient abderaClient = new AbderaClient(abdera);

        String postData = "?importURL=" + sourceURL;
        ByteArrayInputStream bis = new ByteArrayInputStream(postData.getBytes());
        RequestOptions opts = getAuthorization();
        opts.setSlug(suggestedPath);
        opts.setContentType(IMPORT_MEDIATYPE);
        ClientResponse response = abderaClient.post(baseURI + encodeURL(postData), bis, opts);
        return response.getLocation().toString();
    }

    public void delete(String path) throws RegistryException {
        Abdera abdera = new Abdera();
        AbderaClient abderaClient = new AbderaClient(abdera);
        ClientResponse resp = abderaClient.delete(baseURI + encodeURL(path), getAuthorization());
        if (resp.getType() == Response.ResponseType.SUCCESS) {
            log.info(Messages.getMessage("resource.deleted", path));
        } else {
            log.error(Messages.getMessage("resource.delete.fail", path));
            throw new RegistryException(Messages.getMessage("resource.delete.fail", path));
        }
    }


    public String rename(String currentPath, String newPath) throws RegistryException {
        Abdera abdera = new Abdera();
        AbderaClient abderaClient = new AbderaClient(abdera);
        Entry entry = abdera.getFactory().newEntry();
        entry.setContent(newPath);
        ClientResponse resp =
                abderaClient.post(baseURI + encodeURL(currentPath +
                                                      RegistryConstants.URL_SEPARATOR +
                                                      PARAMETER_RENAME),
                                  entry,
                                  getAuthorization());
        if (resp.getType() == Response.ResponseType.SUCCESS) {
            log.info(Messages.getMessage("resource.rename", currentPath));
        } else {
            log.error(Messages.getMessage("rename.resource.fail", currentPath));
            throw new RegistryException(Messages.getMessage("rename.resource.fail", currentPath));
        }
        return newPath;
    }

    public String[] getVersions(String path) throws RegistryException {
        Abdera abdera = new Abdera();
        AbderaClient abderaClient = new AbderaClient(abdera);
        ClientResponse clientResponse =
                abderaClient.get(baseURI + encodeURL(path +
                                                     RegistryConstants.URL_SEPARATOR +
                                                     PARAMETER_VERSION),
                                 getAuthorization());
        Document introspection =
                clientResponse.getDocument();
        Feed feed = (Feed)introspection.getRoot();
        List entries = feed.getEntries();
        if (entries != null) {
            String[] versions = new String[entries.size()];
            for (int i = 0; i < entries.size(); i++) {
                Entry entry = (Entry)entries.get(i);
                versions[i] = entry.getLink("versionLink").getHref().toString();
            }
            return versions;
        }
        return new String[0];
    }

    public void restoreVersion(String versionPath) throws RegistryException {
        Abdera abdera = new Abdera();
        AbderaClient abderaClient = new AbderaClient(abdera);
        Entry entry = abdera.getFactory().newEntry();
        ClientResponse resp = abderaClient.put(baseURI + encodeURL(versionPath +
                                                                   RegistryConstants.URL_SEPARATOR +
                                                                   PARAMETER_RESTORE),
                                               entry,
                                               getAuthorization());
        if (resp.getType() == Response.ResponseType.SUCCESS) {
            log.info(Messages.getMessage("resource.restoreded", versionPath));
        } else {
            log.error(Messages.getMessage("resource.restore.fail", versionPath));
            throw new RegistryException(Messages.getMessage("resource.restore.fail", versionPath));
        }
    }

    public void applyTag(String resourcePath, String tag) throws RegistryException {
        Abdera abdera = new Abdera();
        AbderaClient abderaClient = new AbderaClient(abdera);
        Entry entry = abdera.getFactory().newEntry();
        entry.setContent(tag);
        ClientResponse resp = abderaClient.put(baseURI + encodeURL(resourcePath +
                                                                   RegistryConstants.URL_SEPARATOR +
                                                                   PARAMETER_TAGS),
                                               entry,
                                               getAuthorization());
        if (resp.getType() == Response.ResponseType.SUCCESS) {
            log.info(Messages.getMessage("resource.tagged", resourcePath));
        } else {
            log.error(Messages.getMessage("resource.tag.fails", resourcePath));
            throw new RegistryException(Messages.getMessage("resource.tag.fails", resourcePath));
        }
    }

    public TaggedResourcePath[] getResourcePathsWithTag(String tag) throws RegistryException {
        Abdera abdera = new Abdera();
        AbderaClient abderaClient = new AbderaClient(abdera);
        String taggingUri;
        if (baseURI.endsWith("/")) {
            taggingUri = baseURI;
        } else {
            taggingUri = baseURI + "/";
        }
        ClientResponse clientResponse = abderaClient.get(taggingUri +
                                                         RegistryConstants.URL_SEPARATOR +
                                                         PARAMETER_TAG_PATHS +
                                                         ":" + tag, getAuthorization());
        Document introspection =
                clientResponse.getDocument();
        Feed feed = (Feed)introspection.getRoot();
        List entries = feed.getEntries();
        TaggedResourcePath taggedResourcePaths[] = null;
        if (entries != null) {
            taggedResourcePaths = new TaggedResourcePath[entries.size()];
            for (int i = 0; i < entries.size(); i++) {
                Entry entry = (Entry)entries.get(i);
                Properties properties = entry.getExtension(PropertyExtensionFactory.TAGS);
                List propertyList = properties.getExtensions(PropertyExtensionFactory.TAG);
                Map map = new HashMap();
                for (Object aPropertyList : propertyList) {
                    Property property = (Property)aPropertyList;
                    PropertyName pn = property.getExtension(PropertyExtensionFactory.PROPERTY_NAME);
                    PropertyValue pv =
                            property.getExtension(PropertyExtensionFactory.PROPERTY_VALUE);
                    map.put(pn.getText(), pv.getText());
                }
                TaggedResourcePath tagpath = new TaggedResourcePath();
                tagpath.setResourcePath(entry.getTitle());
                tagpath.setTagCount(
                        Long.parseLong(entry.getSimpleExtension(new QName(NAMESPACE,
                                                                          "taggings"))));
                taggedResourcePaths[i] = tagpath;
                tagpath.setTagCounts(map);
            }
        }
        return taggedResourcePaths;
    }

    public Tag[] getTags(String resourcePath) throws RegistryException {
        Abdera abdera = new Abdera();
        AbderaClient abderaClient = new AbderaClient(abdera);
        ClientResponse clientResponse = abderaClient.get(baseURI +
                                                         encodeURL(resourcePath +
                                                                   RegistryConstants.URL_SEPARATOR +
                                                                   PARAMETER_TAGS),
                                                         getAuthorization());
        Document introspection =
                clientResponse.getDocument();
        Feed feed = (Feed)introspection.getRoot();
        List entries = feed.getEntries();
        Tag tags[] = null;
        if (entries != null) {
            tags = new Tag[entries.size()];
            for (int i = 0; i < entries.size(); i++) {
                Entry entry = (Entry)entries.get(i);
                Tag tag = new Tag();
                tag.setTagCount(Long.parseLong(entry.getSimpleExtension(new QName(NAMESPACE,
                                                                                  "taggings"))));
                tag.setTagName(entry.getTitle());
                tags[i] = tag;
            }
        }
        return tags;
    }

    public void removeTag(String path, String tag) throws RegistryException {
        Abdera abdera = new Abdera();
        AbderaClient abderaClient = new AbderaClient(abdera);
        Entry entry = abdera.getFactory().newEntry();
        entry.setContent(tag);
        ClientResponse resp = abderaClient.put(baseURI + encodeURL(path +
                                                                   RegistryConstants.URL_SEPARATOR +
                                                                   PARAMETER_TAGS_REMOVED),
                                               entry,
                                               getAuthorization());
        if (resp.getType() == Response.ResponseType.SUCCESS) {
            log.info(Messages.getMessage("tag.removed", path));
        } else {
            log.error(Messages.getMessage("tag.removed.fails", path));
            throw new RegistryException(Messages.getMessage("tag.removed.fails", path));
        }
    }

    public String addComment(String resourcePath, Comment comment) throws RegistryException {
        Abdera abdera = new Abdera();
        AbderaClient abderaClient = new AbderaClient(abdera);
        Entry entry = abdera.getFactory().newEntry();
        entry.setUpdated(comment.getTime());
        entry.addAuthor(comment.getUser());
        entry.setContent(comment.getText());
        ClientResponse resp =
                abderaClient.post(baseURI + encodeURL(resourcePath +
                                                      RegistryConstants.URL_SEPARATOR +
                                                      PARAMETER_COMMENTS),
                                  entry,
                                  getAuthorization());
        if (resp.getType() == Response.ResponseType.SUCCESS) {
            log.info(Messages.getMessage("resource.commented", resourcePath));
            return resp.getLocation().toString();
        } else {
            log.error(Messages.getMessage("resouce.comment.fails", resourcePath));
            throw new RegistryException(Messages.getMessage("resouce.comment.fails", resourcePath));
        }
    }


    public void editComment(String commentPath, String text) throws RegistryException {
        Abdera abdera = new Abdera();
        AbderaClient abderaClient = new AbderaClient(abdera);
        Entry entry = abdera.getFactory().newEntry();
        entry.setContent(text);
        ClientResponse resp = abderaClient.put(baseURI + encodeURL(commentPath +
                                                                   RegistryConstants.URL_SEPARATOR +
                                                                   PARAMETER_COMMENTS),
                                               entry,
                                               getAuthorization());
        if (resp.getType() == Response.ResponseType.SUCCESS) {
            log.info(Messages.getMessage("resource.commented", commentPath));
        } else {
            log.error(Messages.getMessage("resource.comment.fails", commentPath));
            throw new RegistryException(Messages.getMessage("resource.comment.fails", commentPath));
        }
    }

    public Comment[] getComments(String resourcePath) throws RegistryException {
        Abdera abdera = new Abdera();
        AbderaClient abderaClient = new AbderaClient(abdera);
        ClientResponse clientResponse =
                abderaClient.get(baseURI + encodeURL(resourcePath +
                                                     RegistryConstants.URL_SEPARATOR +
                                                     PARAMETER_COMMENTS),
                                 getAuthorization());
        Document introspection = clientResponse.getDocument();
        Element element = introspection.getRoot();
        Feed feed = (Feed)element;
        return getCommentsFromFeed(feed);
    }

    private Comment[] getCommentsFromFeed(Feed feed) {
        List entries = feed.getEntries();
        Comment comments[] = null;
        if (entries != null) {
            comments = new Comment[entries.size()];
            for (int i = 0; i < entries.size(); i++) {
                Entry entry = (Entry)entries.get(i);
                Comment comment = new Comment();
                comment.setTime(entry.getUpdated());
                if (entry.getLink("path") != null) {
                    comment.setResourcePath(entry.getLink("path").getHref().toString());
                }
                if (entry.getLink("direct") != null) {
                    comment.setCommentPath(entry.getLink("direct").getHref().toString());
                }
                comment.setText(entry.getContent());
                comment.setUser(entry.getAuthor().getName());
                comments[i] = comment;
            }
        }
        return comments;
    }

    public void rateResource(String resourcePath, int rating) throws RegistryException {
        Abdera abdera = new Abdera();
        AbderaClient abderaClient = new AbderaClient(abdera);
        Entry entry = abdera.getFactory().newEntry();
        entry.setContent("" + rating);
        entry.setSummary(PARAMETER_RATINGS);
        ClientResponse resp = abderaClient.put(baseURI + encodeURL(resourcePath +
                                                                   RegistryConstants.URL_SEPARATOR +
                                                                   PARAMETER_RATINGS),
                                               entry,
                                               getAuthorization());
        if (resp.getType() == Response.ResponseType.SUCCESS) {
            log.info(Messages.getMessage("resource.rated", resourcePath));
        } else {
            log.error(Messages.getMessage("resource.rate.fail", resourcePath));
            throw new RegistryException(Messages.getMessage("resource.rate.fail", resourcePath));
        }
    }

    public float getAverageRating(String resourcePath) throws RegistryException {
        Abdera abdera = new Abdera();
        AbderaClient abderaClient = new AbderaClient(abdera);
        ClientResponse clientResponse =
                abderaClient.get(baseURI + encodeURL(resourcePath +
                                                     RegistryConstants.URL_SEPARATOR +
                                                     PARAMETER_RATINGS),
                                 getAuthorization());

// TODO!!!  Otherwise we get a ClassCastException below...
//        if (clientResponse.getStatus() != 200) {
//            // throw RegistryException
//        }

        Document introspection = clientResponse.getDocument();
        Feed feed = (Feed)introspection.getRoot();
        String floatvalue = feed.getSimpleExtension(QNAME_AVGRATING);
        return Float.parseFloat(floatvalue);
    }

    public int getRating(String path, String userName) throws RegistryException {
        Abdera abdera = new Abdera();
        AbderaClient abderaClient = new AbderaClient(abdera);
        ClientResponse clientResponse =
                abderaClient.get(baseURI + encodeURL(path + RegistryConstants.URL_SEPARATOR +
                                                     PARAMETER_RATINGS + ":" + userName),
                                 getAuthorization());
        Document introspection =
                clientResponse.getDocument();
        Feed feed = (Feed)introspection.getRoot();
        String intvalue = feed.getSubtitle();
        return Integer.parseInt(intvalue);
    }

    public Resource executeQuery(String path, Map parameters) throws RegistryException {
        Abdera abdera = new Abdera();
        AbderaClient abderaClient = new AbderaClient(abdera);
        RequestOptions requestOptions = getAuthorization();
        requestOptions.setHeader("parameters", encodeParametersAsString(parameters));
        ClientResponse resp = abderaClient.get(baseURI + encodeURL(path +
                                                                   RegistryConstants.URL_SEPARATOR +
                                                                   PARAMETER_QUERY),
                                               requestOptions);
        Document introspection =
                resp.getDocument();
        Feed feed = (Feed)introspection.getRoot();
        return createResourceFromFeed(feed, true);
    }

    public static String encodeParametersAsString(Map parameters) {
        String value = "";
        Iterator iParams = parameters.keySet().iterator();
        while (iParams.hasNext()) {
            String paramNumber = (String)iParams.next();
            String paramValue = (String)parameters.get(paramNumber);

            if (iParams.hasNext()) {
                value = value + paramNumber + ":" + paramValue + "#";
            } else {
                value = value + paramNumber + ":" + paramValue;
            }
        }
        return value;
    }

    public static Map decodeParametesAsString(String value) {

        Map paramMap = new HashMap();

        String[] params = value.trim().split("#");
        for (String param : params) {
            if (param.trim().length() == 0) {
                continue;
            }

            String[] paramParts = param.trim().split(":");
            paramMap.put(paramParts[0], paramParts[1]);
        }

        return paramMap;
    }

    public LogEntry[] getLogs(String resourcePath,
                              int action,
                              String userName,
                              Date from,
                              Date to,
                              boolean recentFirst) throws RegistryException {

        Abdera abdera = new Abdera();
        if (resourcePath == null || "".equals(resourcePath)) {
            resourcePath = "/";
        }
        AbderaClient abderaClient = new AbderaClient(abdera);
        RequestOptions requestOptions = getAuthorization();
        requestOptions.addDateHeader("ToDate", to);
        requestOptions.addDateHeader("FromDate", from);
        requestOptions.addHeader("Action", "" + action);
        requestOptions.addHeader("Author", userName);
        ClientResponse resp = abderaClient.get(baseURI + encodeURL(resourcePath +
                                                                   RegistryConstants.URL_SEPARATOR +
                                                                   PARAMETER_LOGS),
                                               requestOptions);
        Document introspection =
                resp.getDocument();
        Feed feed = (Feed)introspection.getRoot();
        List entries = feed.getEntries();
        LogEntry logs[] = null;
        if (entries != null) {
            logs = new LogEntry[entries.size()];
            for (int i = 0; i < entries.size(); i++) {
                Entry entry = (Entry)entries.get(i);
                LogEntry logEntry = new LogEntry();
                logEntry.setDate(entry.getEdited());
                logEntry.setActionData(entry.getContent());
                logEntry.setUserName(entry.getAuthor().getName());
                logEntry.setAction(Integer.parseInt(
                        entry.getSimpleExtension(new QName(NAMESPACE, "action"))));
                logEntry.setResourcePath(entry.getSimpleExtension(new QName(NAMESPACE, "path")));
                logs[i] = logEntry;
            }
        }
        return logs;
    }

    /**
     * This method will create a RequestOptions object adding  Authorization headers.
     *
     * @return RequestOptions : Created RequestOptions object
     */
    private RequestOptions getAuthorization() {
        RequestOptions requestOptions = new RequestOptions();
        requestOptions.setAuthorization(authorizationString);
        return requestOptions;
    }

    public static String encodeURL(String path) {
        //I know I should use URLEncoder but when I used that it tries
        //  to encode "/" as well which is not need here
        return path.replaceAll(" ", "+");
    }

}
