/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */
package org.wso2.registry.config;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.apache.axiom.om.impl.builder.StAXOMBuilder;
import org.wso2.registry.RegistryException;
import org.wso2.registry.i18n.Messages;

import javax.xml.namespace.QName;
import javax.xml.stream.XMLStreamException;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.Iterator;

public class RegistryConfigurationProcessor {
    public static void populateRegistryConfig(String filename, RegistryContext context)
            throws RegistryException {
        try {
            populateRegistryConfig(new FileInputStream(filename), context);
        } catch (FileNotFoundException e) {
            throw new RegistryException(e.getMessage(), e);
        }
    }

    /**
     * Read XML configuration from the passed InputStream, or from the classpath.
     *
     * @param in              an InputStream containing XML data, or null.
     * @param registryContext the RegistryContext to populate
     * @throws RegistryException if there's a problem
     */
    public static void populateRegistryConfig(InputStream in, RegistryContext registryContext)
            throws RegistryException {
        if (in == null) {
            in = Thread.currentThread().getContextClassLoader().getResourceAsStream(
                    "org/wso2/registry/servlet/registry.xml");
        }
        if (in == null) {
            throw new RegistryException(Messages.getMessage("inputstream.null",
                                                            "org/wso2/servlet/registry.xml"));
        }
        try {
            StAXOMBuilder buillder = new StAXOMBuilder(in);
            OMElement configElement = buillder.getDocumentElement();
            if (configElement != null) {

                Iterator dbconfgs = configElement.getChildrenWithName(new QName("dbconfig"));
                while (dbconfgs.hasNext()) {
                    OMElement dbconfig = (OMElement)dbconfgs.next();
                    DataBaseConfiguration dataBaseConfiguration = new DataBaseConfiguration();

                    String dbName = dbconfig.getAttributeValue(new QName("name"));
                    if (dbName == null) {
                        throw new RegistryException(Messages.getMessage("dbname.null"));
                    }
                    dataBaseConfiguration.setConfigName(dbName);
                    OMElement dataSource = dbconfig.getFirstChildWithName(new QName("dataSource"));
                    if (dataSource != null) {
                        dataBaseConfiguration.setDataSourceName(dataSource.getText());

                    } else {

                        OMElement userName = dbconfig.getFirstChildWithName(new QName("userName"));
                        if (userName != null) {
                            dataBaseConfiguration.setUserName(userName.getText());
                        }

                        OMElement password = dbconfig.getFirstChildWithName(new QName("password"));
                        if (password != null) {
                            dataBaseConfiguration.setPassWord(password.getText());
                        }

                        OMElement url = dbconfig.getFirstChildWithName(new QName("url"));
                        if (url != null) {
                            dataBaseConfiguration.setDbUrl(url.getText());
                        }

                        OMElement driverName =
                                dbconfig.getFirstChildWithName(new QName("driverName"));
                        if (driverName != null) {
                            dataBaseConfiguration.setDriverName(driverName.getText());
                        }
                    }

                    registryContext.addDBConfig(dbName, dataBaseConfiguration);
                }

                OMElement currentConfigElement =
                        configElement.getFirstChildWithName(new QName("currentConfig"));
                if (currentConfigElement == null) {
                    throw new RegistryException(Messages.getMessage("db.not.defined"));
                }

                String currentConfigName = currentConfigElement.getText();
                DataBaseConfiguration currentConfiguration =
                        registryContext.getDBConfig(currentConfigName);
                registryContext.setDefaultDataBaseConfiguration(currentConfiguration);

                // process media type handler config
                Iterator mediaTypeHandlers = configElement.
                        getChildrenWithName(new QName("mediaTypeHandler"));
                while (mediaTypeHandlers.hasNext()) {

                    MediaTypeHandlerConfiguration mediaTypeHandlerConfiguration =
                            new MediaTypeHandlerConfiguration();

                    OMElement mediaTypeElement = (OMElement)mediaTypeHandlers.next();
                    OMElement mediaTypeName = mediaTypeElement.
                            getFirstChildWithName(new QName("mediaType"));
                    if (mediaTypeName != null) {
                        mediaTypeHandlerConfiguration.setMediaType(mediaTypeName.getText());
                    }

                    OMElement mediaTypeHandler = mediaTypeElement.
                            getFirstChildWithName(new QName("handler"));
                    if (mediaTypeHandler != null) {
                        mediaTypeHandlerConfiguration.
                                setMediaTypeHandler(mediaTypeHandler.getText());
                    }

                    registryContext.addMediaTypeHandler(mediaTypeHandlerConfiguration);
                }

                // process URL handler config
                Iterator urlHandlers = configElement.getChildrenWithName(new QName("urlHandler"));
                while (urlHandlers.hasNext()) {

                    OMElement urlHandler = (OMElement)urlHandlers.next();
                    registryContext.addURLHandler(urlHandler.getText());
                }

                // process query processor config
                Iterator queryProcessors = configElement.
                        getChildrenWithName(new QName("queryProcessor"));
                while (queryProcessors.hasNext()) {

                    QueryProcessorConfiguration queryProcessorConfiguration =
                            new QueryProcessorConfiguration();

                    OMElement queryProcessorElement = (OMElement)queryProcessors.next();
                    OMElement queryType = queryProcessorElement.
                            getFirstChildWithName(new QName("queryType"));
                    if (queryType != null) {
                        queryProcessorConfiguration.setQueryType(queryType.getText());
                    }

                    OMElement processorName = queryProcessorElement.
                            getFirstChildWithName(new QName("processor"));
                    if (processorName != null) {
                        queryProcessorConfiguration.
                                setProcessorClassName(processorName.getText());
                    }

                    registryContext.addQueryProcessor(queryProcessorConfiguration);
                }
            }

        } catch (XMLStreamException e) {
            throw new RegistryException(e.getMessage());
        }
    }

    public static OMElement getRegistryConfigAsXML(RegistryContext registryContext) {

        OMFactory factory = OMAbstractFactory.getOMFactory();
        OMElement root = factory.createOMElement("wso2registry", null);

        if (registryContext.getDefaultDataBaseConfiguration() != null) {
            OMElement currrentConfigElement = factory.createOMElement("currentConfig", null);
            currrentConfigElement
                    .setText(registryContext.getDefaultDataBaseConfiguration().getConfigName());
            root.addChild(currrentConfigElement);
        }

        Iterator values = registryContext.getDBConfigNames();
        while (values.hasNext()) {
            DataBaseConfiguration dataBaseConfiguration = (DataBaseConfiguration)values.next();
            OMElement config = factory.createOMElement("dbconfig", null);
            OMElement url = factory.createOMElement("url", null);
            url.setText(dataBaseConfiguration.getDbUrl());
            config.addChild(url);
            OMElement userName = factory.createOMElement("userName", null);
            userName.setText(dataBaseConfiguration.getUserName());
            config.addChild(userName);
            OMElement password = factory.createOMElement("password", null);
            password.setText(dataBaseConfiguration.getPassWord());
            config.addChild(password);
            OMElement driverName = factory.createOMElement("driverName", null);
            driverName.setText(dataBaseConfiguration.getDriverName());
            config.addChild(driverName);
            config.addAttribute("name", dataBaseConfiguration.getConfigName(), null);
            root.addChild(config);
        }
        return root;
    }
}
