/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.config;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.RegistryException;
import org.wso2.registry.i18n.Messages;
import org.wso2.registry.jdbc.hsql.DBUtils;
import org.wso2.registry.jdbc.utils.RegistryDataSource;
import org.wso2.registry.secure.HSQLDBInitializer;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.*;

public class RegistryContext {
    public interface RegURLSupplier {
        public String getURL();
    }

    private static final Log log = LogFactory.getLog(RegistryContext.class);

    private RegURLSupplier urlSupplier;
    private DataBaseConfiguration defaultDataBaseConfiguration = null;
    private Map dbConfigs = new HashMap<String, DataBaseConfiguration>();
    private List mediaTypeHandlers = new ArrayList();
    private List urlHandlers = new ArrayList();
    private List queryProcessors = new ArrayList();

    public RegistryContext() {
    }

    public RegistryContext(RegURLSupplier urlSupplier) {
        this.urlSupplier = urlSupplier;
    }

    public DataBaseConfiguration getDefaultDataBaseConfiguration() {
        return defaultDataBaseConfiguration;
    }

    public void setDefaultDataBaseConfiguration(DataBaseConfiguration defaultDataBaseConfiguration) {
        this.defaultDataBaseConfiguration = defaultDataBaseConfiguration;
    }

    public Iterator<String> getDBConfigNames() {
        return dbConfigs.keySet().iterator();
    }

    public DataBaseConfiguration getDBConfig(String dbName) {
        DataBaseConfiguration dataBaseConfiguration = (DataBaseConfiguration)dbConfigs.get(dbName);
        if (dbName.equals("sample")) {
            String dbURL = "jdbc:hsqldb:" + getBasePath() + "/sample/sample-db;shutdown=true";
            dataBaseConfiguration.setDbUrl(dbURL);
            dataBaseConfiguration.setDriverName("org.hsqldb.jdbcDriver");
            dataBaseConfiguration.setUserName("sa");
            dataBaseConfiguration.setPassWord("");

            // create the in-process database

            RegistryDataSource dataSource;
            try {
                dataSource = new RegistryDataSource(
                        dataBaseConfiguration.getDbUrl(),
                        dataBaseConfiguration.getDriverName(),
                        dataBaseConfiguration.getUserName(),
                        dataBaseConfiguration.getPassWord());

                DBUtils dbUtils = new DBUtils();
                Connection conn = dataSource.getConnection();
                dbUtils.initializeDatabase(conn);
                conn.close();

                HSQLDBInitializer dbInitializer = new HSQLDBInitializer();
                dbInitializer.createHSQLTables(dataSource);
            } catch (SQLException e) {
                dataBaseConfiguration.setConfigName("in-memory");
                log.info(Messages.getMessage("unable.to.creat.inmemory"));
            } catch (RegistryException e) {
                dataBaseConfiguration = null;
            }
        }

        return dataBaseConfiguration;
    }

    public void addDBConfig(String name, DataBaseConfiguration config) {
        dbConfigs.put(name, config);
    }

    public List getMediaTypeHandlers() {
        return mediaTypeHandlers;
    }

    public void setMediaTypeHandlers(List mediaTypeHandlers) {
        this.mediaTypeHandlers = mediaTypeHandlers;
    }

    public void addMediaTypeHandler(MediaTypeHandlerConfiguration mediaTypeHandlerConfiguration) {
        mediaTypeHandlers.add(mediaTypeHandlerConfiguration);
    }

    public List getUrlHandlers() {
        return urlHandlers;
    }

    public void setUrlHandlers(List urlHandlers) {
        this.urlHandlers = urlHandlers;
    }

    public void addURLHandler(String urlHandler) {
        urlHandlers.add(urlHandler);
    }

    public List getQueryProcessors() {
        return queryProcessors;
    }

    public void setQueryProcessors(List queryProcessors) {
        this.queryProcessors = queryProcessors;
    }

    public void addQueryProcessor(QueryProcessorConfiguration queryProcessorConfiguration) {
        queryProcessors.add(queryProcessorConfiguration);
    }

    private String getBasePath() {
        String basePath = null;
        if (urlSupplier != null) {
            basePath = urlSupplier.getURL();
        }
        if (basePath == null) {
            basePath = System.getProperty("basedir") + "/target";
        }
        return basePath;
    }
}
