/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */

package org.wso2.registry.inmemory;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.*;
import org.wso2.registry.exceptions.ResourceNotFoundException;
import org.wso2.registry.i18n.Messages;
import org.wso2.registry.inmemory.comments.CommentManager;
import org.wso2.registry.inmemory.ratings.RatingsManager;
import org.wso2.registry.inmemory.tags.TagManager;

import java.util.Date;
import java.util.Map;

/**
 * In memory implementation of the resourceMap. This implementation stores all the resource content
 * and the metadata in the memory as Maps and Lists. Main use case of this implementation is as a
 * cache for a persistance resourceMap.
 */
public class InMemoryRegistry implements Registry {

    protected static final Log log = LogFactory.getLog(InMemoryRegistry.class);

    private TagManager tagManager = new TagManager(this);
    private CommentManager commentManager = new CommentManager(this);
    private RatingsManager ratingManager = new RatingsManager(this);

    /** resource path --> Resource instance */
    private Map resourceMap = new ResourceMap();

    public InMemoryRegistry() {
        Resource root = new Resource();
        root.setPath(RegistryConstants.ROOT_PATH);
        root.setDirectory(true);
        resourceMap.put(RegistryConstants.ROOT_PATH, root);
    }

    public Resource get(String path) throws RegistryException {
        Resource resource = (Resource)resourceMap.get(path);
        if (resource == null) {
            throw new ResourceNotFoundException(path);
        }
        return resource;
    }

    public boolean resourceExists(String path) throws RegistryException {
        return resourceMap.get(path) != null;
    }

    /**
     * Adds or updates resources in the registry. If there is no resource in the given path,
     * resource is added. If a resource already exist in the given path, it will be replaced with
     * the new resource.
     *
     * @param suggestedPath the path which we'd like to use for the new resource.
     * @param resource      Resource instance for the new resource
     * @return the actual path that the server chose to use for our Resource
     * @throws org.wso2.registry.RegistryException
     *
     */
    public synchronized String put(String suggestedPath, Resource resource)
            throws RegistryException {

        resource.setPath(suggestedPath);
        resource.setLastModified(new Date());

        String[] parts = suggestedPath.split(RegistryConstants.PATH_SEPARATOR);

        // check if the new resource is a top level resource
        if (parts.length > 1) {

            // iterate through all parts of the path (by concatenating them), except for the last
            // part (which is the path of the new resource). If directories for those parts do not
            // exists create directories for them. If there is a non-directory resource
            // corresponding to any of those parts, we can't add the resource.
            String currentPath = RegistryConstants.ROOT_PATH;
            Resource currentDirectory = (Resource)resourceMap.get(RegistryConstants.ROOT_PATH);
            for (int i = 1; i < parts.length - 1; i++) {

                // check if a Directory with current Path is already in the resourceMap
                currentPath = (currentPath.equals(RegistryConstants.ROOT_PATH)) ?
                        RegistryConstants.ROOT_PATH +
                        parts[i] : currentPath + RegistryConstants.PATH_SEPARATOR + parts[i];

                Resource r = (Resource)resourceMap.get(currentPath);
                if (r == null) {
                    // there is no Directory for the current path. Let's create and add it to the
                    // resourceMap
                    Resource directoryResource = new Resource();
                    directoryResource.setPath(currentPath);
                    directoryResource.setLastModified(new Date());
                    directoryResource.setDirectory(true);

                    resourceMap.put(currentPath, directoryResource);
                    updateArtifact(currentDirectory, currentPath);
                    currentDirectory = directoryResource;

                } else {
                    if (r.isDirectory()) {
                        currentDirectory = r;
                    } else {
                        String message =
                                Messages.getMessage("resource.delete.error", currentPath,
                                                    suggestedPath);
                        log.error(message);
                        throw new RegistryException(message);
                    }
                }
            }
            updateArtifact(currentDirectory, resource.getPath());
            // now we have the parent directory for the new resource. add the new resource to
            // the resourceMap and make it a child of the parent directory
            resourceMap.put(suggestedPath, resource);

        } else {
            Resource root = (Resource)resourceMap.get(RegistryConstants.ROOT_PATH);
            updateArtifact(root, suggestedPath);
            // new resource is a top level resource. just add it to the resourceMap
            resourceMap.put(suggestedPath, resource);
        }
        return suggestedPath;
    }

    public String importResource(String suggestedPath, String sourceURL, Resource metadata)
            throws RegistryException {
        throw new UnsupportedOperationException(
                "Resource imports are not supported in the InMemoryRegistry.");
    }

    private void updateArtifact(Resource currentDirectory, String currentPath) {
        if (currentDirectory != null && currentDirectory.isDirectory()) {
            String oldPaths[] = (String[])currentDirectory.getContent();
            int count;
            if (oldPaths != null) {
                count = oldPaths.length;
            } else {
                count = 0;
            }
            String newContent[] = new String[count + 1];
            if (count > 0)
                System.arraycopy(oldPaths, 0, newContent, 0, count);
            newContent[count] = currentPath;
            currentDirectory.setContent(newContent);
        }
    }

    /**
     * Deletes a resource from the resourceMap. If the resource is a normal resource, or a link only
     * the resource will be removed. If it is directory, all descendent resources of that directory
     * will be removed.
     * <p/>
     * NOTE: We are not removing links, if a resource is deleted. It will result in run time
     * exception when a link is accessed. But it is better than removing links, which may have
     * created by some other user. Then the user who is using the link can know the exact reason for
     * the invalid link, than going in to some strange error because his link is deleted without his
     * knowledge.
     *
     * @param path Path of the resource to be deleted.
     */
    public synchronized void delete(String path) throws RegistryException {
        Resource resource = (Resource)resourceMap.get(path);
        if (resource == null) {
            String message = Messages.getMessage("resource.delete.error", path);
            log.error(message);
            throw new RegistryException(message);
        }
//        Object o = resource.getContent();
        if (resource.isDirectory()) {
            // remove all descendent resources
            removeDescendents(resource);
            // remove the directory
            removeResource(resource.getPath());
        } else {
            removeResource(resource.getPath());
        }
        // remove all comments of the deleted resource
        commentManager.removeComments(path);
    }


    public String rename(String currentPath, String newPath) throws RegistryException {
        throw new UnsupportedOperationException();
    }

    public String[] getVersions(String path) throws RegistryException {
        throw new UnsupportedOperationException(Messages.getMessage("unsupported.exception",
                                                                    "Versioning"));
    }

    public void restoreVersion(String versionPath) throws RegistryException {
        throw new UnsupportedOperationException(Messages.getMessage("unsupported.exception",
                                                                    "Versioning"));
    }

    public synchronized void applyTag(String resourcePath, String tag) throws RegistryException {
        tagManager.applyTag(resourcePath, tag);
    }

    public TaggedResourcePath[] getResourcePathsWithTag(String tag) throws RegistryException {
        return tagManager.getResourcePathsWithTag(tag);
    }

    public Tag[] getTags(String resourcePath) throws RegistryException {
        return tagManager.getTags(resourcePath);
    }

    public void removeTag(String path, String tag) throws RegistryException {
        throw new UnsupportedOperationException(Messages.getMessage("unsupported.exception",
                                                                    "Removing tags"));
    }

    public synchronized String addComment(String resourcePath, Comment comment)
            throws RegistryException {
        return resourcePath + ";comments:" + commentManager.addComment(resourcePath, comment);
    }


    public void editComment(String commentPath, String text) throws RegistryException {
        throw new UnsupportedOperationException(Messages.getMessage("unsupported.exception",
                                                                    "edit comments"));
    }

    public Comment[] getComments(String resourcePath) throws RegistryException {
        return commentManager.getComments(resourcePath);
    }

    public synchronized void rateResource(String resourcePath, int rating)
            throws RegistryException {
        ratingManager.rateResource(resourcePath, rating);
    }

    public float getAverageRating(String resourcePath) throws RegistryException {
        return ratingManager.getAverageRating(resourcePath);
    }

    public int getRating(String path, String userName) throws RegistryException {
        throw new UnsupportedOperationException(Messages.getMessage("unsupported.exception",
                                                                    "User level rating"));
    }

    public Resource executeQuery(String path, Map parameters) throws RegistryException {
        throw new UnsupportedOperationException(Messages.getMessage("unsupported.exception",
                                                                    "Custom queries"));
    }

    public synchronized boolean isResourceAvailable(String path) {
        return resourceMap.get(path) != null;
    }

    /**
     * Removes descendents (child directories and resources) from the given directory.
     *
     * @param resource Resource object with a CollectionContent
     */
    private void removeDescendents(Resource resource) {

        String[] childPaths = (String[])resource.getContent();
        if (childPaths == null) {
            return;
        }

        for (String childPath : childPaths) {
            Resource childResource = (Resource)resourceMap.get(childPath);

            if (childResource.isDirectory()) {

                removeDescendents(childResource);
                removeResource(childResource.getPath());

            } else {
                removeResource(childResource.getPath());
            }
        }
    }

    /**
     * Removes the given resource, by performing necessary clean up.
     *
     * @param resourcePath Path of the resource to be removed.
     */
    private void removeResource(String resourcePath) {

        // clean up all meta data for the resource
        tagManager.removeTaggings(resourcePath);
        commentManager.removeComments(resourcePath);
        ratingManager.removeRatings(resourcePath);

        // actually remove the resource
        resourceMap.remove(resourcePath);
    }

    public LogEntry[] getLogs(String resourcePath,
                              int action,
                              String userName,
                              Date from,
                              Date to,
                              boolean recentFirst)
            throws RegistryException {
        throw new UnsupportedOperationException(Messages.getMessage("unsupported.exception",
                                                                    "Logs"));
    }

}
