/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */

package org.wso2.registry.inmemory.tags;

import org.wso2.registry.RegistryException;
import org.wso2.registry.Tag;
import org.wso2.registry.TaggedResourcePath;
import org.wso2.registry.i18n.Messages;
import org.wso2.registry.inmemory.InMemoryRegistry;

import java.util.*;

public class TagManager {

    private InMemoryRegistry inMemoryRegistryProvider = null;

    /**
     * resourcePath --> tag This is the primary map to store taggings. We may store Tagging
     * instances instead of String tags to capture tagged user and time in future.
     */
    private Map resources = new HashMap();

    /**
     * tag --> resourcePath This is a helper map to increase the speed of searches by tags. We don't
     * need to change String tag key, even though the resources map chaned to store Tagging
     * objects.
     */
    private Map tags = new HashMap();

    public TagManager(InMemoryRegistry inMemoryRegistryProvider) {
        this.inMemoryRegistryProvider = inMemoryRegistryProvider;
    }

    public void applyTag(String resourcePath, String tag) throws RegistryException {

        if (!inMemoryRegistryProvider.isResourceAvailable(resourcePath)) {
            throw new RegistryException(Messages.getMessage("cannot.add.tag", resourcePath));
        }

        // add tagging to the tags map
        List resourceList = (List)tags.get(tag);

        if (resourceList == null) {
            // this tag has not been used before. so add it to the map
            resourceList = new ArrayList();
            tags.put(tag, resourceList);
        }
        resourceList.add(resourcePath);

        // add tagging to the resources map
        List tagList = (List)resources.get(resourcePath);

        if (tagList == null) {
            tagList = new ArrayList();
            resources.put(resourcePath, tagList);
        }
        tagList.add(tag);
    }

    public TaggedResourcePath[] getResourcePathsWithTag(String tag) {

        List resourceList = (List)tags.get(tag);

        if (resourceList == null) {
            return new TaggedResourcePath[0];
        }

        TaggedResourcePath[] taggedPaths = new TaggedResourcePath[resourceList.size()];
        for (int i = 0; i < taggedPaths.length; i++) {
            String resourcePath = (String)resourceList.get(i);
            TaggedResourcePath taggedResourcePath = new TaggedResourcePath();
            taggedResourcePath.setResourcePath(resourcePath);
            taggedResourcePath.setTagCount(1);
            taggedPaths[i] = taggedResourcePath;
        }

        return taggedPaths;
    }

    public Tag[] getTags(String resourcePath) throws RegistryException {

        //throw new UnsupportedOperationException("Retrieving tags for a resource path " +
        //        "is not supported in the HashMap based in-memory registry.");

        if (!inMemoryRegistryProvider.isResourceAvailable(resourcePath)) {
            throw new RegistryException(Messages.getMessage("resource.get.error", resourcePath));
        }

        List tagList = (List)resources.get(resourcePath);

        if (tagList == null) {
            return new Tag[0];
        }

        Tag[] tags = new Tag[tagList.size()];
        for (int i = 0; i < tags.length; i++) {
            String tagName = (String)tagList.get(i);
            Tag tag = new Tag();
            tag.setTagName(tagName);
            tag.setTagCount(1);
            tags[i] = tag;
        }

        return tags;
    }

    public void removeTaggings(String resourcePath) {

        resources.remove(resourcePath);

        for (Iterator i1 = tags.values().iterator(); i1.hasNext();) {
            Object o1 = i1.next();

            if (o1 != null) {
                List taggedResources = (List)o1;
                for (int i = 0; i < taggedResources.size(); i++) {
                    String rp1 = (String)taggedResources.get(i);

                    if (rp1.equals(resourcePath)) {
                        taggedResources.remove(rp1);
                    }
                }
            }
        }

    }
}
