/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */

package org.wso2.registry.jdbc;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.RegistryException;
import org.wso2.registry.i18n.Messages;
import org.wso2.registry.jdbc.hsql.DBUtils;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;

/**
 * Manages the database connections of the JDBC registry. This can be configured on 2 ways.
 * <p/>
 * 1) Using a datasource Data source name has to be specified in the constructor. If the default
 * constructor is used "java:comp/env/jdbc/WSO2RegistryDB" is used as the data source name.
 * <p/>
 * 2) Using a database connection URL
 * <p/>
 * In either case, we can specify whether this should use HSQL in-memory database in case of a
 * failure of above methods. If in-memory database is allowed, relevant hsql jars should in the
 * class path.
 */
public class ConnectionFactory {

    private static final Log log = LogFactory.getLog(ConnectionFactory.class);

    private String dataSourceName = null;
    private String connectionURL = null;
    private String userName = null;
    private String password = null;
    private boolean allowInMemoryDatabase = true;

    private DataSource dataSource = null;

    public ConnectionFactory() throws RegistryException {
        this.dataSourceName = DatabaseConstants.REGISTRY_DATASOURCE_NAME;
        this.dataSource = getDataSource();
    }

    public ConnectionFactory(String dataSourceName) throws RegistryException {
        this.dataSourceName = dataSourceName;
        this.dataSource = getDataSource();
    }

    public ConnectionFactory(String driverClass, String url, String userName, String password)
            throws RegistryException {

        try {
            Class.forName(driverClass);

        } catch (ClassNotFoundException e) {
            String msg = "Database driver class " + driverClass +
                         " could not be loaded. Caused by: " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }

        this.connectionURL = url;
        this.userName = userName;
        this.password = password;
    }

    public boolean isAllowInMemoryDatabase() {
        return allowInMemoryDatabase;
    }

    public void setAllowInMemoryDatabase(boolean allowInMemoryDatabase) {
        this.allowInMemoryDatabase = allowInMemoryDatabase;
    }

    public DataSource getDataSource() throws RegistryException {

        if (dataSource != null) {
            return dataSource;
        }

        try {
            log.info("Trying to find the data source " + dataSourceName + "...");

            Context context = new InitialContext();
            DataSource dataSource = (DataSource)context.lookup(dataSourceName);

            log.info("Found the data source " + dataSourceName + ".");
            return dataSource;

        } catch (NamingException e) {
            log.info(Messages.
                    getMessage("datasource.not.found", dataSourceName));


            if (allowInMemoryDatabase) {
                log.info("Starting the in-memory database...");

                startInProcessDatabase();

                log.info("Successfully started the in-memory database.");
            }
        }
        return null;
    }

    public Connection getConnection() throws RegistryException {

        Connection conn = null;

        if (dataSource != null) {
            try {
                conn = dataSource.getConnection();
            } catch (SQLException e) {
                String message = Messages.getMessage("database.error");
                throw new RegistryException(message);
            }

        } else {

            try {

                if (connectionURL != null) {
                    conn = DriverManager.getConnection(connectionURL, userName, password);

                } else if (allowInMemoryDatabase) {
                    conn = DriverManager.getConnection("jdbc:hsqldb:mem:aname", "sa", "");
                }

                //conn = DriverManager.getConnection("jdbc:hsqldb:file:/home/chathura/temp/hsqldb", "sa", "");

                //String dbURL = "jdbc:mysql://localhost:3306/test?user=root";
                //conn = DriverManager.getConnection(dbURL, new Properties());

            } catch (SQLException e) {
                throw new RegistryException("Couldn't get HSQL connection.");
            }
        }

        return conn;
    }

    private void startInProcessDatabase() throws RegistryException {

        // start hsql or embedded derby as the Registry database

        // this is used only if a data source is not configured for the environment
        // this should only be used in test cases and demos

        try {
            //Class.forName("com.mysql.jdbc.Driver");
            Class.forName("org.hsqldb.jdbcDriver");
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }

        // Create tables
        try {
            Connection conn = DriverManager.getConnection("jdbc:hsqldb:mem:aname", "sa", "");
            new DBUtils().initializeDatabase(conn);

        } catch (SQLException e) {
            String message = Messages.getMessage("database.error");
            throw new RegistryException(message);
        }
    }

    public void shutdown() throws Exception {
        //Statement s = getConnection().createStatement();
        //s.execute("SHUTDOWN");
    }
}
