/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */

package org.wso2.registry.jdbc;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.*;
import org.wso2.registry.exceptions.ResourceNotFoundException;
import org.wso2.registry.i18n.Messages;
import org.wso2.registry.jdbc.dao.*;
import org.wso2.registry.jdbc.mediatypes.MediaTypeManager;
import org.wso2.registry.jdbc.queries.QueryProcessorManager;
import org.wso2.registry.jdbc.realm.RegistryRealm;
import org.wso2.registry.jdbc.urlhandlers.URLHandlerManager;
import org.wso2.registry.utils.AuthorizationUtil;
import org.wso2.usermanager.Realm;

import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.*;

/**
 * JDBC based implementation of the Registry. This will be used mostly as the back-end by other
 * Registry implementations. This can use either an in-memory database or a external database
 * configured using data source. At the initializing, it checks if a data source named
 * "jdbc/WSO2RegistryDB" is configured. If such data source is found, it is used as the database. If
 * a data source is not found, an in-memory database is created using the HSQL.
 */
public class JDBCRegistry implements Registry {

    private static final Log log = LogFactory.getLog(JDBCRegistry.class);

    private DataSource dataSource = null;
    private URLHandlerManager urlHandlerManager;
    private MediaTypeManager mediaTypeManager;
    private QueryProcessorManager queryProcessorManager;
    private VersionedResourceDAO resourceDAO = null;
    private TagsDAO tagsDAO = null;
    private CommentsDAO commentsDAO = null;
    private RatingsDAO ratingsDAO = null;
    private LogsDAO logsDAO = null;

    private Realm defaultRealm = null;

    /**
     * Default constructor. JDBC registry should be configured using the configure(...) method, if
     * it is instantiated using this constructor.
     */
    public JDBCRegistry() {
    }

    /**
     * Constructs a JDBC registry to be used without any security (authorizations on actions). JDBC
     * registry instances created using this constructor should not be used with SecureRegistry
     * instances.
     *
     * @param dataSource Data source to be used to store resources and metadata.
     * @throws RegistryException : If something went wrong while init
     */
    public JDBCRegistry(DataSource dataSource) throws RegistryException {
        this.dataSource = dataSource;
        init(null);
    }

    /**
     * Constructs a JDBC registry to be used with secure registries. Default authorizations will be
     * applied on the resources created using this registry.
     *
     * @param dataSource Data source to be used to store resources and metadata. Note that the same
     *                   data source may or may not be used as the data source of user manager.
     * @param realm      User manager realm to handle authorizations. It is strongly recommended to
     *                   use a realm instance constructed using the RegistryRealmFactory. All
     *                   initial users, roles and authorizations will be set on realms obtained
     *                   using the RegistryRealmFactory.
     * @throws RegistryException : If something went wrong
     */
    public JDBCRegistry(DataSource dataSource, Realm realm) throws RegistryException {
        this.dataSource = dataSource;
        init(realm);
    }

    /**
     * Configures and initiates the JDBC registry with a (new) datasource and a realm. This is
     * useful for changing underlying databases at run-time.
     *
     * @param dataSource the DataSource to use
     * @param realm      Realm to use
     * @throws RegistryException : If something went wrong while init
     */
    public void configure(DataSource dataSource, RegistryRealm realm) throws RegistryException {
        this.dataSource = dataSource;
        init(realm);
    }

    private void init(Realm realm) throws RegistryException {

        defaultRealm = realm;

        urlHandlerManager = new URLHandlerManager(dataSource, realm, this);
        mediaTypeManager = new MediaTypeManager(dataSource, realm);
        queryProcessorManager = new QueryProcessorManager(dataSource, realm);

        resourceDAO = new VersionedResourceDAO();
        tagsDAO = new TagsDAO();
        commentsDAO = new CommentsDAO();
        ratingsDAO = new RatingsDAO();
        logsDAO = new LogsDAO();

        // check if root is added. if not add it.
        Connection conn = getConnection();

        try {
            conn.setAutoCommit(false);

            Resource root = resourceDAO.getLatestVersion(RegistryConstants.ROOT_PATH, conn);
            if (root == null) {
                root = new Resource();
                root.setPath(RegistryConstants.ROOT_PATH);
                root.setDirectory(true);
                root.setAuthorUserName(RegistryConstants.SYSTEM_USER);
                root.setLastUpdaterUserName(RegistryConstants.SYSTEM_USER);
                resourceDAO.add(root, conn);

                AuthorizationUtil.setDefaultAuthorizations(defaultRealm,
                                                           RegistryConstants.ROOT_PATH,
                                                           RegistryConstants.SYSTEM_USER);

                root = resourceDAO.getLatestVersion(RegistryConstants.ROOT_PATH, conn);
                root.setLastUpdaterUserName(RegistryConstants.SYSTEM_USER);
                resourceDAO.addResourceVersion(root, conn);
            }

            conn.commit();

        } catch (Exception e) {

            try {
                conn.rollback();
            } catch (SQLException e1) {
                e1.printStackTrace();
            }

            String msg = Messages.getMessage("jdbc.reg.init.fail");
            log.error(msg, e);
            throw new RegistryException(msg, e);

        } finally {

            try {
                conn.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }
    }

    public Realm getDefaultRealm() {
        return defaultRealm;
    }

    /**
     * Returns the resource in the given path.
     *
     * @param path Path of the resource. e.g. /project1/server/deployment.xml
     * @return Resource instance
     * @throws RegistryException ResourceNotFound exception is thrown if the resource is not in the
     *                           registry
     */
    public Resource get(String path) throws RegistryException {

        path = preparePath(path);
        Resource resource;

        // check if this path refers to a resource referred by a URL query (e.g. comment)
        resource = urlHandlerManager.get(path);

        // check if the resource in the path has a mediatype handler
        if (resource == null) {
            resource = mediaTypeManager.get(path);
        }

        if (resource == null) {
            throw new ResourceNotFoundException(path);
        }

        // if the request is for the current version of the resource, it should not be in the
        // deleted state.
        if (path.indexOf("?v=") == -1 && resource.getState() == RegistryConstants.DELETED_STATE) {
            throw new ResourceNotFoundException(path);
        }

        return resource;
    }

    public boolean resourceExists(String path) throws RegistryException {

        boolean exists = false;

        Connection conn = getConnection();
        try {
            exists = resourceDAO.resourceExist(path, conn);
        } catch (SQLException e) {
            String msg = Messages.getMessage("faild.to.check", path);
            log.error(msg, e);
            throw new RegistryException(msg, e);

        } finally {
            try {
                if (conn != null) conn.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }

        return exists;
    }

    /**
     * Adds or updates resources in the registry. If there is no resource in the given path,
     * resource is added. If a resource already exist in the given path, it will be replaced with
     * the new resource. Implementations may use a concurrency control method to prevent concurrency
     * issues.
     *
     * @param suggestedPath the path which we'd like to use for the new resource.
     * @param resource      Resource instance for the new resource
     * @return the actual path that the server chose to use for our Resource
     * @throws org.wso2.registry.RegistryException
     *
     */
    public synchronized String put(String suggestedPath, Resource resource)
            throws RegistryException {
        suggestedPath = preparePath(suggestedPath);

        String actualPath;

        actualPath = urlHandlerManager.put(suggestedPath, resource);

        if (actualPath == null) {
            mediaTypeManager.put(suggestedPath, resource);
        }

        Connection conn = getConnection();

        try {
            conn.setAutoCommit(false);

            long resourceID = resourceDAO.getResourceID(suggestedPath, conn);
            logsDAO.addLog(resourceID, User.getCurrentUser(), LogEntry.UPDATE, null, conn);

            conn.commit();

        } catch (SQLException e) {
            try {
                conn.rollback();
            } catch (SQLException e1) {
                e1.printStackTrace();
            }

            String msg = e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);

        } finally {
            try {
                conn.close();
            } catch (SQLException ignore) {
                log.info(Messages.getMessage("exception.closing.db"));
            }
        }

        if (actualPath == null) {
            return suggestedPath;
        } else {
            return actualPath;
        }
    }

    public String importResource(String suggestedPath, String sourceURL, Resource metadata)
            throws RegistryException {

        suggestedPath = preparePath(suggestedPath);

        String savedPath = mediaTypeManager.importResource(suggestedPath, sourceURL, metadata);

        Connection conn = getConnection();

        try {
            conn.setAutoCommit(false);

            long resourceID = resourceDAO.getResourceID(savedPath, conn);

            if (resourceID != -1) {
                logsDAO.addLog(resourceID, User.getCurrentUser(), LogEntry.UPDATE, null, conn);
            }

            conn.commit();

        } catch (SQLException e) {
            try {
                conn.rollback();
            } catch (SQLException e1) {
                e1.printStackTrace();
            }

            String msg = e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);

        } finally {
            try {
                conn.close();
            } catch (SQLException ignore) {
                log.info(Messages.getMessage("exception.closing.db"));
            }
        }

        return suggestedPath;
    }


    public String rename(String currentPath, String newPath) throws RegistryException {

        if (currentPath == null || currentPath.length() == 0 ||
            newPath == null || newPath.length() == 0) {
            String msg = "Invlaid resource paths. Current path: " +
                         currentPath + ". New path: " + newPath;
            throw new RegistryException(msg);
        }

        // if the current path and the new resource path are same, we don't want to do anything
        if (currentPath.equals(newPath)) {
            return currentPath;
        }

        String userID = User.getCurrentUser();

        Connection conn = getConnection();

        if (resourceExists(currentPath)) {

            if (!resourceExists(newPath)) {

                try {

                    long resourceID = resourceDAO.getResourceID(currentPath, conn);

                    resourceDAO.renameResource(currentPath,
                                               newPath,
                                               getConnection(),
                                               User.getCurrentUser(),
                                               defaultRealm);

                    logsDAO.addLog(resourceID, userID, LogEntry.RENAME, currentPath, conn);

                } catch (SQLException e) {

                    String msg = "Failed to rename the resource: " +
                                 currentPath + " to the name: " + newPath;
                    log.error(msg, e);
                    throw new RegistryException(msg, e);

                } finally {
                    if (conn != null) {
                        try {
                            conn.close();
                        } catch (SQLException ignore) {
                            log.info(Messages.getMessage("exception.closing.db"));
                        }
                    }
                }
            }
            return newPath;
        } else {
            throw new ResourceNotFoundException(currentPath);
        }
    }

    /**
     * Deletes the resource in the given path. If the path refers to a directory, all child
     * resources of the directory will also be deleted.
     *
     * @param path Path of the resource to be deleted.
     * @throws org.wso2.registry.RegistryException
     *          : if the user is not authorized
     */
    public synchronized void delete(String path) throws RegistryException {

        path = preparePath(path);

        if (!urlHandlerManager.delete(path)) {
            mediaTypeManager.delete(path);
        }

        Connection conn = getConnection();

        try {
            conn.setAutoCommit(false);

            long resourceID = resourceDAO.getResourceID(path, conn);
            logsDAO.addLog(resourceID, User.getCurrentUser(), LogEntry.DELETE_RESOURCE, null, conn);

            conn.commit();

        } catch (SQLException e) {
            try {
                conn.rollback();
            } catch (SQLException e1) {
                e1.printStackTrace();
            }

            String msg = e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);

        } finally {
            try {
                conn.close();
            } catch (SQLException ignore) {
                log.info(Messages.getMessage("exception.closing.db"));
            }
        }
    }

    public String[] getVersions(String path) throws RegistryException {

        String[] versionPaths = null;

        Connection conn = getConnection();

        try {
            long artifactID = resourceDAO.getResourceID(path, conn);
            String[] versionIDs = resourceDAO.getVersionNumbers(artifactID, conn);

            versionPaths = new String[versionIDs.length];
            for (int i = 0; i < versionIDs.length; i++) {
                versionPaths[i] = path + "?v=" + versionIDs[i];
            }

        } catch (SQLException e) {

            try {
                conn.rollback();
            } catch (SQLException e1) {
                e1.printStackTrace();
            }
            String msg = Messages.getMessage("could.not.found.versionpath", path, e.getMessage());
            log.error(msg, e);
            throw new RegistryException(msg, e);

        } finally {
            try {
                conn.close();
            } catch (SQLException ignore) {
                log.info("Nothing we can do here : Inside the finally block ");
            }
        }

        return versionPaths;
    }

    private Connection getConnection() throws RegistryException {
        Connection conn;
        try {
            conn = dataSource.getConnection();
        } catch (SQLException e) {
            throw new RegistryException(Messages.getMessage("unable.to.open.db"), e);
        }
        return conn;
    }

    public void restoreVersion(String versionPath) throws RegistryException {
        String plainPath;
        long versionNumber;
        if (versionPath.indexOf("?v=") != -1) {
            String[] parts = versionPath.split("\\?v=");
            plainPath = parts[0];
            versionNumber = Long.parseLong(parts[1]);
        } else {
            String msg = Messages.getMessage("not.a.valid.verion.path");
            log.error(msg);
            throw new RegistryException(msg);
        }

        String userID = User.getCurrentUser();

        Connection conn = getConnection();

        try {
            conn.setAutoCommit(false);

            long artifactID = resourceDAO.getResourceID(plainPath, conn);

            resourceDAO.restore(artifactID, versionNumber, conn);

            logsDAO.addLog(
                    artifactID, userID, LogEntry.RESTORE, Long.toString(versionNumber), conn);

            conn.commit();

        } catch (SQLException e) {
            String msg = Messages.getMessage("unable.to.restore", versionPath, e.getMessage());
            log.error(msg, e);

            try {
                conn.rollback();
            } catch (SQLException e1) {
                log.error(e1);
            }

            throw new RegistryException(msg, e);

        } finally {
            try {
                conn.close();
            } catch (SQLException ignore) {
                log.info(Messages.getMessage("exception.closing.db"));
            }
        }
    }

    ////////////////////////////////////////////////////////
    // Tagging
    ////////////////////////////////////////////////////////

    /**
     * Applies the given tag to the resource in the given path. If the given tag is not defined in
     * the registry, it will be defined.
     *
     * @param resourcePath Path of the resource to be tagged.
     * @param tag          Tag. Any string can be used for the tag.
     * @throws RegistryException ResourceNotFound exception is thrown if a resource does not exist
     *                           in the given path.
     */
    public synchronized void applyTag(String resourcePath, String tag) throws RegistryException {

        resourcePath = preparePath(resourcePath);

        Connection conn = getConnection();

        String userID = User.getCurrentUser();

        // break the comma separated words into multiple tags
        String[] tags = tag.split(",");

        try {
            conn.setAutoCommit(false);

            if (!resourceDAO.resourceExist(resourcePath, conn)) {
                throw new RegistryException(
                        "Cannot tag on a non-existing resource " + resourcePath);
            }
            Resource resource = resourceDAO.getLatestVersion(resourcePath, conn);

            for (int i = 0; i < tags.length; i++) {

                tags[i] = tags[i].trim();

                if (tags[i].length() == 0 || tags[i].equals(" ")) {
                    continue;
                }

                if (tagsDAO.taggingExist(tags[i], resource.getId(), userID, conn)) {
                    // Already there, don't bother doing it again.
                    return;
                }

                tagsDAO.addTagging(tags[i], resource.getId(), userID, conn);
                logsDAO.addLog(resource.getId(), userID, LogEntry.TAG, tags[i], conn);
            }

            conn.commit();

        } catch (SQLException e) {
            try {
                conn.rollback();
            } catch (SQLException e1) {
                e1.printStackTrace();
            }

            String msg = Messages.getMessage("resource.tagging.fail", resourcePath, tag);
            log.error(msg, e);
            throw new RegistryException(msg, e);

        } finally {
            try {
                conn.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * Returns the paths of the resources, which are tagged using the given tag.
     *
     * @param tag Tag
     * @return Resource paths
     */
    public TaggedResourcePath[] getResourcePathsWithTag(String tag) throws RegistryException {

        // break the tags from spaces
        String[] tags = tag.trim().split(",");

        for (int i = 0; i < tags.length; i++) {
            tags[i] = tags[i].trim();
        }

        //List pathList = new ArrayList();
        List taggedPaths = new ArrayList();

        Connection conn = getConnection();
        try {
            List pathList = tagsDAO.getPathsWithAnyTag(tags, conn);
            Iterator iPaths = pathList.iterator();
            while (iPaths.hasNext()) {
                String path = (String)iPaths.next();

                TaggedResourcePath taggedResourcePath = new TaggedResourcePath();
                taggedResourcePath.setResourcePath(path);

                for (String tag1 : tags) {
                    long count = tagsDAO.getTagCount(path, tag1, conn);
                    taggedResourcePath.addTagCount(tag1, count);
                }

                taggedPaths.add(taggedResourcePath);
            }

        } catch (SQLException e) {

            String msg = Messages.getMessage("tag.search.fail.1", tag);
            log.error(msg, e);
            throw new RegistryException(msg, e);

        } finally {
            try {
                conn.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }

        return
                (TaggedResourcePath[])taggedPaths
                        .toArray(new TaggedResourcePath[taggedPaths.size()]);
    }

    /**
     * Returns all tags used for tagging the given resource.
     *
     * @param resourcePath Path of the resource
     * @return Tags
     * @throws RegistryException ResourceNotFound exception is thrown if a resource does not exist
     *                           in the given path.
     */
    public Tag[] getTags(String resourcePath) throws RegistryException {

        resourcePath = preparePath(resourcePath);
        resourcePath = getCurrentPath(resourcePath);

        Connection conn = getConnection();

        Tag[] tags = null;

        try {
            tags = tagsDAO.getTagsWithCount(resourcePath, conn);
        } catch (SQLException e) {

            String msg = Messages.getMessage("tag.search.fail.2", resourcePath);
            log.error(msg, e);
            throw new RegistryException(msg, e);

        } finally {
            try {
                conn.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }

        return tags;
    }

    public void removeTag(String path, String tag) throws RegistryException {

        path = preparePath(path);

        String user = User.getCurrentUser();

        Connection conn = getConnection();

        try {
            conn.setAutoCommit(false);

            Resource taggedResource = resourceDAO.getLatestVersion(path, conn);
            if (user.equals(taggedResource.getAuthorUserName())) {
                tagsDAO.removeTag(path, tag, conn);
            } else {
                tagsDAO.removeTag(path, tag, user, conn);
            }

            conn.commit();

        } catch (SQLException e) {

            String msg = "Could not remove the tag: " + tag + " from the resource at path: " +
                         path + ".";
            log.error(msg, e);

            try {
                conn.rollback();
            } catch (SQLException e1) {
                e1.printStackTrace();
            }

            throw new RegistryException(msg, e);

        } finally {
            try {
                conn.close();
            } catch (SQLException ignore) {
                log.info(Messages.getMessage("exception.closing.db"));
            }
        }
    }

    ////////////////////////////////////////////////////////
    // Comments
    ////////////////////////////////////////////////////////

    /**
     * Adds a comment to a resource.
     *
     * @param resourcePath Path of the resource to add the comment.
     * @param comment      Comment instance for the new comment.
     * @throws RegistryException ResourceNotFound exception is thrown if a resource does not exist
     *                           in the given path.
     */
    public synchronized String addComment(String resourcePath, Comment comment)
            throws RegistryException {

        resourcePath = preparePath(resourcePath);

        Connection conn = getConnection();

        String userID = User.getCurrentUser();
        String commentPath;
        try {
            conn.setAutoCommit(false);

            Resource resource = resourceDAO.getLatestVersion(resourcePath, conn);
            if (resource != null) {
                commentPath = commentsDAO.addComment(resource.getId(), userID, comment, conn);
                commentPath = resourcePath + ";comments:" + commentPath;
                logsDAO.addLog(
                        resource.getId(), userID, LogEntry.COMMENT, comment.getText(), conn);
            } else {

                String msg = Messages.getMessage("comment.on.null.artfact", resourcePath);
                log.error(msg);
                throw new RegistryException(msg);
            }

            conn.commit();

        } catch (SQLException e) {

            try {
                conn.rollback();
            } catch (SQLException e1) {
                e1.printStackTrace();
            }

            String msg = Messages.
                    getMessage("resource.comment.fail", resourcePath, comment.getText());
            log.error(msg, e);
            throw new RegistryException(msg, e);

        } finally {

            try {
                conn.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }
        return commentPath;
    }


    public void editComment(String commentPath, String text) throws RegistryException {
        String[] parts = commentPath.split(";");
        String commentPart = parts[1];
        if (parts.length == 2 && commentPart.startsWith("comments:")) {
            String commentId = parts[1].substring(9);
            try {
                commentsDAO.updateComment(Long.parseLong(commentId), text, getConnection());
            } catch (SQLException e) {
                //TODO : i18n
            }
        }
    }

    /**
     * Returns all comments for the given resource.
     *
     * @param resourcePath Path of the resource.
     * @return Array of Comment objects.
     */
    public Comment[] getComments(String resourcePath) throws RegistryException {

        resourcePath = preparePath(resourcePath);
        resourcePath = getCurrentPath(resourcePath);

        Connection conn = getConnection();
        try {
            return commentsDAO.getComments(resourcePath, conn);
        } catch (SQLException e) {

            String msg = Messages.getMessage("get.comments.fail", resourcePath);
            log.error(msg, e);
            throw new RegistryException(msg, e);

        } finally {
            try {
                conn.close();
            } catch (SQLException e) {
                log.info(Messages.getMessage("exception.closing.db"));
            }
        }
    }

    ////////////////////////////////////////////////////////
    // Ratings
    ////////////////////////////////////////////////////////

    /**
     * Rates the given resource.
     *
     * @param resourcePath Path of the resource.
     * @param rating       rating value.
     * @throws RegistryException ResourceNotFound exception is thrown if a resource does not exist
     *                           in the given path.
     */
    public synchronized void rateResource(String resourcePath, int rating)
            throws RegistryException {

        resourcePath = preparePath(resourcePath);

        Connection conn = getConnection();

        String userID = User.getCurrentUser();

        try {
            conn.setAutoCommit(false);

            Resource resource = resourceDAO.getLatestVersion(resourcePath, conn);

            if (resource != null) {
                if (ratingsDAO.ratingExist(resource.getId(), userID, conn)) {
                    ratingsDAO.updateRating(resource.getId(), userID, rating, conn);
                    log.info("Updated the rating on the resource " +
                             resourcePath + " by user " + userID);
                } else {
                    ratingsDAO.addRating(resource.getId(), userID, rating, conn);
                }

                logsDAO.addLog(resource.getId(),
                               userID, LogEntry.RATING, Integer.toString(rating), conn);

            } else {
                String msg = Messages.getMessage("rate.on.null.artfact", resourcePath);
                log.error(msg);
                throw new RegistryException(msg);
            }

            conn.commit();

        } catch (SQLException e) {

            try {
                conn.rollback();
            } catch (SQLException e1) {
                e1.printStackTrace();
            }

            String msg = Messages.getMessage("resource.rating.fail", resourcePath);
            log.error(msg, e);
            throw new RegistryException(msg, e);

        } finally {

            try {
                conn.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }


    }

    /**
     * Returns the average rating (from all users) for the given resource.
     *
     * @param resourcePath Path of the resource.
     * @return average rating.
     */
    public float getAverageRating(String resourcePath) throws RegistryException {

        resourcePath = preparePath(resourcePath);
        resourcePath = getCurrentPath(resourcePath);

        Connection conn = getConnection();

        float rating = 0;

        try {
            rating = ratingsDAO.getAverageRating(resourcePath, conn);
        } catch (SQLException e) {

            String msg = Messages.getMessage("get.average.rating.fail", resourcePath);
            log.error(msg, e);
            throw new RegistryException(msg, e);

        } finally {
            try {
                conn.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }

        return rating;
    }

    public int getRating(String path, String userName) throws RegistryException {

        path = preparePath(path);
        path = getCurrentPath(path);

        Connection conn = getConnection();

        int rating = 0;

        try {
            rating = ratingsDAO.getRating(path, userName, conn);
        } catch (SQLException e) {

            String msg = "Could not get the rating of the resource " + path +
                         " given by the user " + userName + ". Caused by: " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);

        } finally {
            try {
                conn.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }

        return rating;
    }

    //////////////////////////////////////////////////////////////////////////////
    // Extensible searching API
    //////////////////////////////////////////////////////////////////////////////

    public synchronized void defineQuery(String name, String query) throws RegistryException {

        Connection conn = getConnection();

        String userID = User.getCurrentUser();

        // make the path of the query (e.g. /users/chathura/queries/myquery1 )
        String queryPath = RegistryConstants.ROOT_PATH + RegistryConstants.USERS_COLLECTION_NAME +
                           RegistryConstants.PATH_SEPARATOR + userID + RegistryConstants
                .PATH_SEPARATOR +
                                RegistryConstants.QUERIES_COLLECTION_NAME + RegistryConstants
                .PATH_SEPARATOR + name;


        try {
            conn.setAutoCommit(false);

            Resource queryResource = new Resource();
            queryResource.setPath(queryPath);
            queryResource.setMediaType(RegistryConstants.SQL_QUERY_MEDIA_TYPE);
            queryResource.setAuthorUserName(userID);
            queryResource.setContent(query.getBytes());

            Resource resource = resourceDAO.getLatestVersion(queryPath, conn);
            if (resource != null) {
                resourceDAO.update(queryPath, queryResource, conn);
                log.info("Updated the predefined query: " + queryPath);
            } else {
                resourceDAO.add(queryResource, conn);
            }

            conn.commit();

        } catch (SQLException e) {

            try {
                conn.rollback();
            } catch (SQLException e1) {
                e1.printStackTrace();
            }

            String msg = Messages.getMessage("define.query.fail", name, query);
            log.error(msg, e);
            throw new RegistryException(msg, e);

        } finally {
            try {
                conn.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }


    }

    public Resource executeQuery(String path, Map parameters) throws RegistryException {

        Connection conn = getConnection();

        try {
            Resource query = get(path);
            //QueryProcessor queryProcessor =
            //        new QueryProcessorManager().getQueryProcessor(query.getMediaType());
            //if (queryProcessor == null) {
            //    throw new RegistryException("Undefined query type: " + query.getMediaType());
            //}
            return queryProcessorManager.executeQuery(query, parameters);
        } finally {
            try {
                conn.close();
            } catch (SQLException ignore) {
                log.info(Messages.getMessage("exception.closing.db"));
            }
        }
    }

    public void shutdown() throws RegistryException {

    }

    private String getCurrentPath(String resourcePath) {
        String currentPath = resourcePath;
        if (resourcePath.indexOf("?") > 0) {
            currentPath = resourcePath.split("\\?")[0];
        } else if (resourcePath.indexOf(";") > 0) {
            currentPath = resourcePath.split("\\;")[0];
        }

        return currentPath;
    }

    private String preparePath(String rawPath) {

        String path = rawPath;

        // path should always start with "/"
        if (!rawPath.startsWith(RegistryConstants.ROOT_PATH)) {
            path = RegistryConstants.ROOT_PATH + rawPath;
        }

        return path;
    }

    public LogEntry[] getLogs(String resourcePath,
                              int action,
                              String userName,
                              Date from, Date to, boolean recentFirst) throws RegistryException {

        Connection conn = getConnection();

        List logEntryList;

        try {
            logEntryList =
                    logsDAO.getLogs(resourcePath, action, userName, from, to, recentFirst, conn);

        } catch (SQLException e) {
            String msg = "Could not get logs. Caused by: " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);

        } finally {
            try {
                conn.close();
            } catch (SQLException ignore) {
                log.info(Messages.getMessage("exception.closing.db"));
            }
        }
        LogEntry[] logEntries = new LogEntry[logEntryList.size()];
        for (int i = 0; i < logEntryList.size(); i++) {
            logEntries[i] = (LogEntry)logEntryList.get(i);
        }

        return logEntries;
    }

}
