/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.mediatypes.builtin;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.RegistryException;
import org.wso2.registry.Resource;
import org.wso2.registry.User;
import org.wso2.registry.i18n.Messages;
import org.wso2.registry.jdbc.mediatypes.MediaTypeHandler;
import org.wso2.registry.jdbc.mediatypes.MediaTypeManager;
import org.wso2.registry.utils.AuthorizationUtil;
import org.wso2.usermanager.Realm;

import javax.sql.DataSource;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Iterator;
import java.util.List;

public class DefaultMediaTypeHandler extends MediaTypeHandler {

    private static final Log log = LogFactory.getLog(DefaultMediaTypeHandler.class);

    public DefaultMediaTypeHandler(DataSource dataSource,
                                   Realm realm,
                                   MediaTypeManager mediaTypeManager) {

        super(dataSource, realm, mediaTypeManager);
    }

    public Resource get(String path, Resource resource) throws RegistryException {

        String plainPath = path;
        long versionNumber = -1;
        if (path.indexOf("?v=") != -1) {
            String[] parts = path.split("\\?v=");
            plainPath = parts[0];
            versionNumber = Long.parseLong(parts[1]);
        }

        plainPath = getPreparedPath(plainPath);

        Connection conn;
        try {
            conn = dataSource.getConnection();
        } catch (SQLException e) {
            throw new RegistryException(e.getMessage());
        }
        //if we uncomment following three lines then , no onc access a deleted resource
//        if (!resourceDAO.isResourceActive(plainPath,conn)) {
//             throw new RegistryException("Trying to access a deleted resource");
//        }
        try {
            //resource = resourceDAO.get(path, conn);
            if (versionNumber == -1) {
                resource = resourceDAO.getLatestVersion(plainPath, conn);

            } else {
                resource = resourceDAO.get(plainPath, versionNumber, conn);
            }

        } catch (SQLException e) {

            String msg = Messages.getMessage("resource.get.fail", path);
            log.error(msg, e);
            throw new RegistryException(msg);

        } finally {
            try {
                conn.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }

        // we are just getting the content from the database. not a content modification.
        if (resource != null) {
            resource.setContentModified(false);
        }

        return resource;
    }

    public boolean put(String path, Resource resource) throws RegistryException {

        String userID = User.getCurrentUser();

        Connection conn;
        try {
            conn = dataSource.getConnection();
        } catch (SQLException e) {
            throw new RegistryException(e.getMessage());
        }

        try {
            conn.setAutoCommit(false);

            Resource a = resourceDAO.getLatestVersion(path, conn);
            if (a != null) {

                // if the parent collections of this resource are in deleted state, make them
                // activated
                activateParentCollections(path, conn);

                // given resource already exist. update it.
                resource.setId(a.getId());

                // if client has not explicitly set the author, keep the original author name.
                if (resource.getAuthorUserName() == null) {
                    resource.setAuthorUserName(a.getAuthorUserName());
                }
                resource.setPath(path);
                resource.setState(RegistryConstants.ACTIVE_STATE);
                resource.setLastUpdaterUserName(userID);
                resourceDAO.update(path, resource, conn);
                if (resource.isDirectory()) {
                    if (!a.isDirectory()) {
                        throw new RegistryException("Trying to add a collection while a " +
                                                    "resource with the same name exist in the system");
                    }
                } else {
                    if (a.isDirectory()) {
                        throw new RegistryException("Trying to add a resource while a " +
                                                    "collection with the same name exist in the system");
                    }
                }
                if (resource.isContentModified() ||
                    a.getState() == RegistryConstants.DELETED_STATE) {
                    resourceDAO.addResourceVersion(resource, conn);
                }

            } else {

                // given resource does not exist. add it as a new resource.

                // first create all ascendant collections
                createParentCollections(path, conn, userID);

                // now add the required resource
                resource.setPath(path);
                if (resource.getAuthorUserName() == null) {
                    resource.setAuthorUserName(userID);
                }
                resource.setLastUpdaterUserName(userID);

                resourceDAO.add(resource, conn);
                AuthorizationUtil.setDefaultAuthorizations(realm, path, userID);

                resource.setId(resourceDAO.getResourceID(path, conn));
                resourceDAO.addResourceVersion(resource, conn);
                // add new resource as a child of the current parent
                //resourceDAO.addChild(currentParent.getId(), resource.getId(), conn);
            }

            conn.commit();

        } catch (SQLException e) {

            try {
                conn.rollback();
            } catch (SQLException e1) {
                e1.printStackTrace();
            }

            String msg = Messages.getMessage("resource.put.fail", path, e.getMessage());
            log.error(msg, e);
            throw new RegistryException(msg);
        } finally {
            try {
                conn.close();
            } catch (SQLException e) {
                log.error("Exception occur in side the finaly block of " + this + " put");
            }
        }
        return true;
    }

    /**
     * This method will first split the given path and then try to create directory for each part
     * for example if the path is "/abc/xyx/r1" , then first it will see whether the resource "/abc"
     * is there if it is not there then it will create that else go to "/abc/xyz" , and see whether
     * the resource is there , if so continue the same procedure
     *
     * @param path   : Location where we need to add the resource
     * @param conn   : DB connection
     * @param userID : user
     * @throws SQLException      : Something went wrong
     * @throws RegistryException : Something went wrong
     */
    private void createParentCollections(String path,
                                         Connection conn,
                                         String userID) throws SQLException, RegistryException {
        String[] parts = path.split(RegistryConstants.PATH_SEPARATOR);

        String currentPath = RegistryConstants.ROOT_PATH + parts[1];
        for (int i = 2; i < parts.length; i++) {

            Resource currentCollection = resourceDAO.getLatestVersion(currentPath, conn);
            if (currentCollection == null) {

                currentCollection = new Resource();
                currentCollection.setPath(currentPath);
                currentCollection.setDirectory(true);
                currentCollection.setAuthorUserName(userID);
                currentCollection.setLastUpdaterUserName(userID);

                resourceDAO.add(currentCollection, conn);
                AuthorizationUtil.setDefaultAuthorizations(realm, currentPath, userID);
//                        currentCollection = resourceDAO.getLatestVersion(currentPath, conn);

            } else {

                if (currentCollection.getState() == RegistryConstants.DELETED_STATE) {

                    //currentCollection = new Resource();
                    //currentCollection.setPath(currentPath);
                    //currentCollection.setDirectory(true);
                    //currentCollection.setAuthorUserName(userID);
                    //currentCollection.setLastUpdaterUserName(userID);
                    resourceDAO.markActivated(currentCollection.getId(), conn);
                    //put(currentPath, currentCollection);

                } else if (!currentCollection.isDirectory()) {

                    String message =
                            Messages.getMessage("resource.adding.error", currentPath, path);
                    throw new RegistryException(message);
                }
            }

            currentPath = currentPath + RegistryConstants.PATH_SEPARATOR + parts[i];
        }
    }

    private void activateParentCollections(String path,
                                           Connection conn) throws SQLException, RegistryException {

        if (path.equals(RegistryConstants.ROOT_PATH)) {
            // if we are at root, there are no parent collections.
            return;
        }

        String[] parts = path.split(RegistryConstants.PATH_SEPARATOR);

        String currentPath = RegistryConstants.ROOT_PATH + parts[1];
        for (int i = 2; i < parts.length; i++) {

            Resource currentCollection = resourceDAO.getLatestVersion(currentPath, conn);
            if (currentCollection == null) {

                String msg = "Parent collection: " + currentPath + " of the exsiting resource:" +
                             path + " cannot be null";
                log.error(msg);
                throw new RegistryException(msg);

            } else {

                if (currentCollection.getState() == RegistryConstants.DELETED_STATE) {

                    resourceDAO.markActivated(currentCollection.getId(), conn);

                } else if (!currentCollection.isDirectory()) {

                    String msg = "Parent : " + currentPath + " of the exsiting resource:" + path +
                                 " should be a collection resource.";
                    log.error(msg);
                    throw new RegistryException(msg);
                }
            }

            currentPath = currentPath + RegistryConstants.PATH_SEPARATOR + parts[i];
        }
    }

    public String importResource(String path, String sourceURL, Resource metadata)
            throws RegistryException {

        URL url;
        try {
            url = new URL(sourceURL);
        } catch (MalformedURLException e) {
            String msg = "Given source URL is not valid.";
            throw new RegistryException(msg, e);
        }

        try {
            URLConnection uc = url.openConnection();
            InputStream in = uc.getInputStream();

            ByteArrayOutputStream out = new ByteArrayOutputStream();
            byte[] buffer = new byte[1024];
            int c;
            while ((c = in.read(buffer)) != -1) {
                out.write(buffer, 0, c);
            }
            out.flush();

            Resource resource = new Resource();
            resource.setPath(path);
            String mediaType = metadata.getMediaType();
            if (mediaType == null)
                mediaType = uc.getContentType();
            resource.setMediaType(mediaType);
            resource.setDescription(metadata.getDescription());
            resource.setContent(out.toByteArray());

            put(path, resource);

        } catch (IOException e) {
            String msg = "Could not read from the given URL: " + sourceURL;
            throw new RegistryException(msg, e);
        }
        return path;
    }

    public boolean delete(String path) throws RegistryException {

        Connection conn;
        try {
            conn = dataSource.getConnection();
        } catch (SQLException e) {
            throw new RegistryException(e.getMessage());
        }

        try {
            conn.setAutoCommit(false);
            Resource resource = resourceDAO.getLatestVersion(path, conn);
            if (resource != null) {
                if (resource.isDirectory()) {
                    deleteDirectory(resource, conn);
                } else {
                    resourceDAO.markDeleted(path, conn);
                    resourceDAO.deleteResource(resource, conn);
                    //prepareToDelete(resource.getId(), conn);
                    //resourceDAO.delete(resource.getResourcePath(), conn);
                }
            } else {
                String msg = Messages.getMessage("delete.null.resource", path);
                log.error(msg);
                throw new RegistryException(msg);
            }
            conn.commit();
        } catch (SQLException e) {
            try {
                conn.rollback();
            } catch (SQLException e1) {
                log.error("Exception occur in side the catch block of " + this + " delete");
            }
            String msg = Messages.getMessage("resource.delete.fail", path);
            log.error(msg, e);
            throw new RegistryException(msg);
        } finally {
            try {
                conn.close();
            } catch (SQLException e) {
                log.error("Exception occur in side the finaly block of " + this + " delete");
            }
        }
        return true;
    }

    private void deleteDirectory(Resource collection, Connection conn)
            throws SQLException, RegistryException {
        List childList = resourceDAO.getChildren(collection.getId(), -1, conn);
        Iterator i = childList.iterator();
        while (i.hasNext()) {
            Resource child = (Resource)i.next();
//            if (child.isDirectory()) {
//                deleteDirectory(child, conn);
//            } else {
            resourceDAO.markDeleted(child.getPath(), conn);
            resourceDAO.deleteResource(child, conn);
//            }
        }
        resourceDAO.markDeleted(collection.getPath(), conn);
        resourceDAO.deleteResource(collection, conn);
    }

    public boolean putChild(String childPath, Resource resource) throws RegistryException {
        // General resource should not have any constraint on adding children.
        return true;
    }

    public boolean importChild(String childPath, String sourceURL) throws RegistryException {
        return true;
    }

    private String getPreparedPath(String resourcePath) {

        String preparedPath = resourcePath;
        if (preparedPath.equals(RegistryConstants.ROOT_PATH)) {
            return preparedPath;

        } else {

            if (!preparedPath.startsWith(RegistryConstants.ROOT_PATH)) {
                preparedPath = RegistryConstants.ROOT_PATH + preparedPath;
            }

            if (preparedPath.endsWith(RegistryConstants.PATH_SEPARATOR)) {
                preparedPath = preparedPath.substring(0, preparedPath.length() - 1);
            }
        }

        return preparedPath;
    }
}
