/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.mediatypes.utils;

import org.apache.ws.commons.schema.XmlSchema;
import org.apache.ws.commons.schema.XmlSchemaCollection;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.RegistryException;
import org.wso2.registry.Resource;
import org.wso2.registry.jdbc.mediatypes.builtin.DefaultMediaTypeHandler;

import javax.wsdl.Definition;
import javax.wsdl.Import;
import javax.wsdl.Types;
import javax.wsdl.WSDLException;
import javax.wsdl.extensions.schema.Schema;
import javax.wsdl.factory.WSDLFactory;
import javax.wsdl.xml.WSDLReader;
import javax.wsdl.xml.WSDLWriter;
import java.io.ByteArrayOutputStream;
import java.util.*;

public class WSDLFileProcessor {

    // remove this when it is not needed
    private int i;

    public static final String IMPORT_TAG = "import";
    public static final String INCLUDE_TAG = "include";

    DefaultMediaTypeHandler defaultMediaTypeHandler;

    public WSDLFileProcessor(DefaultMediaTypeHandler defaultMediaTypeHandler) {
        this.defaultMediaTypeHandler = defaultMediaTypeHandler;
    }

    /**
     * saves a wsdl file including its imports and imported schemas.
     *
     * @param location
     * @throws WSDLException
     */
    public String saveWSDLFileToRegistry(String location,
                                         String registryBasePath,
                                         boolean processImports,
                                         Resource metadata)
            throws RegistryException {
        WSDLReader wsdlReader;
        String fileNameToSave;

        try {
            wsdlReader = WSDLFactory.newInstance().newWSDLReader();
        } catch (WSDLException e) {
            String msg = "Could not initiate the wsdl reader. Caused by: " + e.getMessage();
            throw new RegistryException(msg);
        }

        wsdlReader.setFeature("javax.wsdl.importDocuments", true);
        Definition wsdlDefinition;

        try {
            wsdlDefinition = wsdlReader.readWSDL(location);
            String baseUri = wsdlDefinition.getDocumentBaseURI();
            String wsdlFileName = baseUri.substring(baseUri.lastIndexOf("/") + 1);
            fileNameToSave = wsdlFileName.substring(0, wsdlFileName.indexOf(".")) + ".wsdl";
        } catch (WSDLException e) {
            String msg = "Could not read the wsdl at location " + location + ". Caused by: " +
                         e.getMessage();
            throw new RegistryException(msg);
        }

        Map processedWSDLMap = new HashMap();
        calculateWSDLNamesAndChangeTypes(wsdlDefinition, processedWSDLMap, new HashMap(),
                                         new HashSet(), registryBasePath, processImports);
        saveWSDLFileToRegistry(wsdlDefinition, processedWSDLMap, new HashSet(), registryBasePath,
                               processImports, metadata);

        return fileNameToSave;
    }

    /**
     * saves the given wsdl definition file with its imported wsdls and imported and included
     * schemas.
     *
     * @param wsdlDefinition     - wsdl file to save
     * @param processedWSDLMap   - map with original source URI vs new uris for wsdls
     * @param processedScheamMap - map with orignal source URI vs new uris for schemas
     */

    public void calculateWSDLNamesAndChangeTypes(Definition wsdlDefinition,
                                                 Map processedWSDLMap,
                                                 Map processedScheamMap,
                                                 Set visitedWSDLs,
                                                 String registryBasePath,
                                                 boolean processImports) throws RegistryException {
        // first we have to process the imports and change the
        // schema locations suite for the registry

        if (processImports) {
            Iterator iter = wsdlDefinition.getImports().values().iterator();
            Vector values;
            Import wsdlImport;
            // add this to visited list to stop recursion
            visitedWSDLs.add(wsdlDefinition.getDocumentBaseURI());
            for (; iter.hasNext();) {
                values = (Vector)iter.next();
                for (Iterator valuesIter = values.iterator(); valuesIter.hasNext();) {
                    wsdlImport = (Import)valuesIter.next();
                    // process the types recuresiveilt
                    Definition innerDefinition = wsdlImport.getDefinition();
                    if (!visitedWSDLs.contains(innerDefinition.getDocumentBaseURI())) {
                        // we have not process this wsdl file earlier
                        calculateWSDLNamesAndChangeTypes(
                                innerDefinition, processedWSDLMap, processedScheamMap, visitedWSDLs,
                                registryBasePath, processImports);
                    }
                }
            }
        }

        // change the schema names
        Types types = wsdlDefinition.getTypes();
        if (types != null) {
            List extensibleElements = types.getExtensibilityElements();
            Schema schemaExtension;
            Object extensionObject;
            XmlSchema xmlSchema;
            XmlSchemaCollection xmlSchemaCollection;
            SchemaFileProcessor schemaFileProcessor;
            Map changedLocationMap;
            String basuri = wsdlDefinition.getDocumentBaseURI();
            basuri = basuri.substring(0, basuri.lastIndexOf("/") + 1);
            for (Iterator extensibleElementsIter = extensibleElements.iterator();
                 extensibleElementsIter.hasNext();) {
                extensionObject = extensibleElementsIter.next();
                if (extensionObject instanceof Schema) {
                    // first get the schema object
                    schemaExtension = (Schema)extensionObject;
                    // create an xml schema object to be processed by SchemaFile procesor.
                    xmlSchemaCollection = new XmlSchemaCollection();
                    xmlSchemaCollection.setBaseUri(basuri);
                    xmlSchema = xmlSchemaCollection.read(schemaExtension.getElement());
                    schemaFileProcessor = new SchemaFileProcessor(defaultMediaTypeHandler);
                    changedLocationMap = new HashMap();
                    schemaFileProcessor.calculateNewSchemaNames(
                            xmlSchema, processedScheamMap, new HashSet(), true, processImports);
                    schemaFileProcessor.saveSchemaFileToRegistry(
                            xmlSchema, processedScheamMap, changedLocationMap, new HashSet(), true,
                            registryBasePath, processImports, null);
                    // update the current schema locations with the generated ones.
                    changeLocations(schemaExtension.getElement(), changedLocationMap);
                }
            }
        }

        // after processing the defintions save this to the registry
        // TODO: save this to the registry for the moment save this to the
        // folder and omit any exception occurs.
        String baseUri = wsdlDefinition.getDocumentBaseURI();
        String wsdlFileName = baseUri.substring(baseUri.lastIndexOf("/") + 1);
        String fileNameToSave = wsdlFileName.substring(0, wsdlFileName.indexOf(".")) + ".wsdl";
        while (processedWSDLMap.containsValue(fileNameToSave)) {
            fileNameToSave = wsdlFileName.substring(0, wsdlFileName.indexOf(".")) + ++i + ".wsdl";
        }
        // add this entry to the proccessed wsdl map
        processedWSDLMap.put(baseUri, fileNameToSave);

    }

    public void saveWSDLFileToRegistry(Definition wsdlDefinition,
                                       Map processedWSDLMap,
                                       Set visitedWSDLs,
                                       String registryBasePath,
                                       boolean processImports,
                                       Resource metadata) throws RegistryException {

        if (processImports) {
            // first we have to process the imports and change the
            // schema locations suite for the registry
            Iterator iter = wsdlDefinition.getImports().values().iterator();
            Vector values;
            Import wsdlImport;
            // add this to visited list to stop recursion
            visitedWSDLs.add(wsdlDefinition.getDocumentBaseURI());
            for (; iter.hasNext();) {
                values = (Vector)iter.next();
                for (Iterator valuesIter = values.iterator(); valuesIter.hasNext();) {
                    wsdlImport = (Import)valuesIter.next();
                    // process the types recuresiveilt
                    Definition innerDefinition = wsdlImport.getDefinition();
                    if (!visitedWSDLs.contains(innerDefinition.getDocumentBaseURI())) {
                        // we have not process this wsdl file earlier
                        saveWSDLFileToRegistry(innerDefinition, processedWSDLMap, visitedWSDLs,
                                               registryBasePath, processImports, null);
                    }
                    // set the import location according to the new location
                    wsdlImport.setLocationURI((String)processedWSDLMap.get(
                            innerDefinition.getDocumentBaseURI()));
                }
            }
        }

        // after processing the defintions save this to the registry
        // save this to the registry
        String importedResourceName =
                (String)processedWSDLMap.get(wsdlDefinition.getDocumentBaseURI());
        try {
            WSDLWriter wsdlWriter = WSDLFactory.newInstance().newWSDLWriter();
            ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
            wsdlWriter.writeWSDL(wsdlDefinition, byteArrayOutputStream);
            byte[] wsdlResourceContent = byteArrayOutputStream.toByteArray();

            // create a resource this wsdlResourceContent and put it to the registry with the name importedResourceName (in some path)

            String wsdlPath;
            if (RegistryConstants.ROOT_PATH.equals(registryBasePath)) {
                wsdlPath = RegistryConstants.ROOT_PATH + importedResourceName;
            } else {
                wsdlPath =
                        registryBasePath + RegistryConstants.PATH_SEPARATOR + importedResourceName;
            }
            Resource wsdlResource = new Resource();

            if (metadata != null) {
                wsdlResource.setMediaType(metadata.getMediaType());
                wsdlResource.setDescription(metadata.getDescription());
            }
            wsdlResource.setContent(wsdlResourceContent);
            defaultMediaTypeHandler.put(wsdlPath, wsdlResource);

            //wsdlWriter.writeWSDL(wsdlDefinition, new FileOutputStream("repository/" + importedResourceName));
        } catch (WSDLException e) {
            e.printStackTrace();
        }
    }

    private void changeLocations(Element element, Map changedLocationMap) {
        NodeList nodeList = element.getChildNodes();
        String tagName;
        for (int i = 0; i < nodeList.getLength(); i++) {
            tagName = nodeList.item(i).getLocalName();
            if (IMPORT_TAG.equals(tagName) || INCLUDE_TAG.equals(tagName)) {
                processImport(nodeList.item(i), changedLocationMap);
            }
        }
    }

    private void processImport(Node importNode, Map changedLocationMap) {
        NamedNodeMap nodeMap = importNode.getAttributes();
        Node attribute;
        String attributeValue;
        for (int i = 0; i < nodeMap.getLength(); i++) {
            attribute = nodeMap.item(i);
            if (attribute.getNodeName().equals("schemaLocation")) {
                attributeValue = attribute.getNodeValue();
                if (changedLocationMap.get(attributeValue) != null) {
                    attribute.setNodeValue(
                            (String)changedLocationMap.get(attributeValue));
                }
            }
        }
    }

}
