/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.urlhandlers;

import org.wso2.registry.RegistryException;
import org.wso2.registry.Resource;
import org.wso2.usermanager.Realm;

import javax.sql.DataSource;

/**
 * Base class for URL handlers. URL handlers are used support virtual resources. JDBC registry
 * allows users to access some none-resource entities in the same way they access normal resources
 * (e.g. comments, tags, ratings). These entities can be accessed by providing a special URL to
 * identify the virtual resource.
 * <p/>
 * For example tags for the resource at path /projects/mybank/design can be accessed by the URL
 * /projects/mybank/design?tags. URL handlers are used to support that behavior. That is, we can
 * register URL handlers to handle each type of virtual resource. Then those URL handlers are
 * responsible for interpreting the special URL and generating the virtual resource referred by that
 * URL.
 */
public abstract class URLHandler {

    /**
     * Datasource of the registry database. URL handlers can access this to contruct resources
     * by combining various tables (e.g. comments).
     */
    protected DataSource dataSource;

    /**
     * Default realm of the user manager used in the registry. This can be used by the URL handler
     * implementations to check authorizations on resources.
     */
    protected Realm realm;

    public URLHandler(DataSource dataSource, Realm realm) {
        this.dataSource = dataSource;
        this.realm = realm;
    }

    protected URLHandler() {
    }

    /**
     * Determine if get(...) on the given url can be handled. If yes, process the request and return
     * the resulting resource. Further processing of the resource is terminated if a resource is
     * returned. If the given url cannot be handled return null, which causes other url handlers to
     * process the request, or the request to be reached the media type handlers.
     *
     * @param url URL to identify the resource to be retrieved.
     *
     * @return Resource instance if the URL can be processed. Null if the request cannot be
     *         processed.
     *
     * @throws RegistryException URL handler implementations should handle all exceptions and
     * throw RegistryException if the exception has to propagated to the client.
     */
    public Resource get(String url) throws RegistryException {
        return null;
    }

    /**
     * Determine if put(...) on the given url can be handled. If yes, process the request and
     * persisting the resulting resource. Further processing of the resource is terminated if string
     * (actual path) is returned. If the given url cannot be handled return null, which causes other
     * url handlers to process the request, or the request to be reached the media type handlers.
     *
     * @param suggestPath Suggested path to store the resource.
     *
     * @param resource Resource to be stored.
     *
     * @return If the URL handler impl processes the put request, it returns the actual path
     * on which resource is stored. If it does not process the resource, null should be returned
     * indicating that the request has to be propagated to the next handler.
     *
     * @throws RegistryException URL handler implementations should handle all exceptions and
     * throw RegistryException if the exception has to propagated to the client.
     */
    public String put(String suggestPath, Resource resource) throws RegistryException {
        return null;
    }

    /**
     * Determine if delete(...) on the given url can be handled. If yes, process the request and
     * delete the resulting resource. Further processing of the resource is terminated if true is
     * returned. If the given url cannot be handled return false, which causes other url handlers to
     * process the request, or the request to be reached the media type handlers.
     *
     * @param path Path of the resource to be deleted.
     *
     * @return true if the URL handler impl processes the delete request. false if it does not
     * process the delete request. 
     *
     * @throws RegistryException URL handler implementations should handle all exceptions and
     * throw RegistryException if the exception has to propagated to the client.
     */
    public boolean delete(String path) throws RegistryException {
        return false;
    }
}
