/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */

package org.wso2.registry.secure;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.RegistryException;
import org.wso2.usermanager.*;
import org.wso2.usermanager.acl.realm.AuthorizingRealm;
import org.wso2.usermanager.acl.realm.AuthorizingRealmConfig;

import java.util.Map;

/**
 * Wrapper to handle user store related operations in the registry. Main task of this class is to
 * protect the set of predefined users, roles and authorizations from the actions of registry users
 * (e.g. it should NOT be possible to remove system user, admin user or anonymous user, it should
 * NOT be possible to remove or deny authorizations of system or admin users). Therefore, all user
 * manager related operations should be accessed through an instance of this class.
 */
public class RegistryUserManager {

    private static final Log log = LogFactory.getLog(RegistryUserManager.class);

    private Realm realm;
    private AccessControlAdmin accessControlAdmin;
    private Authorizer authorizer;
    private UserStoreAdmin userStoreAdmin;
    private UserStoreReader userStoreReader;

    public RegistryUserManager(Realm userRealm) throws RegistryException {

        this.realm = userRealm;

        try {
            accessControlAdmin = userRealm.getAccessControlAdmin();
            authorizer = userRealm.getAuthorizer();
            userStoreAdmin = userRealm.getUserStoreAdmin();
            userStoreReader = userRealm.getUserStoreReader();

        } catch (UserManagerException e) {

            String msg =
                    "Could not get sub interfaces from the realm. Caused by: " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }
    }

    public RegistryUserManager(Realm deafuleRealm, String userName) throws RegistryException {

        try {

            this.realm = new AuthorizingRealm();
            AuthorizingRealmConfig config = (AuthorizingRealmConfig)this.realm
                    .getRealmConfiguration();
            config.setRealm(deafuleRealm);
            config.setAuthenticatedUserName(userName);
            this.realm.init(config);

            accessControlAdmin = deafuleRealm.getAccessControlAdmin();
            authorizer = deafuleRealm.getAuthorizer();
            userStoreAdmin = deafuleRealm.getUserStoreAdmin();
            userStoreReader = deafuleRealm.getUserStoreReader();

        } catch (UserManagerException e) {

            String msg = "Could not initiate the authorizing realm. Caused by: " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }
    }

    public Realm getRealm() {
        return realm;
    }

    public void addUser(String userName,
                        String password) throws RegistryException {

        try {
            userStoreAdmin.addUser(userName, password);
            userStoreAdmin.addUserToRole(userName, RegistryConstants.EVERYONE_ROLE);

        } catch (UserManagerException e) {
            String msg = "Could not add the user. \nCaused by " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }
    }

    public void addRole(String roleName) throws RegistryException {

        try {
            userStoreAdmin.addRole(roleName);
        } catch (UserManagerException e) {
            String msg = "Could not add the role. \nCaused by " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }
    }

    public void removeUser(String userName) throws RegistryException {

        if (userName.equals(RegistryConstants.ADMIN_USER)) {
            String msg = "Cannot remove the administrator.";
            log.info(msg);
            throw new RegistryException(msg);
        }

        if (userName.equals(RegistryConstants.SYSTEM_USER)) {
            String msg = "Cannot remove the system user.";
            log.info(msg);
            throw new RegistryException(msg);
        }

        if (userName.equals(RegistryConstants.ANONYMOUS_USER)) {
            String msg = "Cannot remove the anonymous user.";
            log.info(msg);
            throw new RegistryException(msg);
        }

        try {
            userStoreAdmin.deleteUser(userName);
        } catch (UserManagerException e) {
            String msg = "Could not delete the user. \nCaused by " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }
    }

    public void removeRole(String roleName) throws RegistryException {

        if (roleName.equals(RegistryConstants.ADMIN_ROLE)) {
            String msg = "Cannot remove the administrator role.";
            log.info(msg);
            throw new RegistryException(msg);
        }

        if (roleName.equals(RegistryConstants.EVERYONE_ROLE)) {
            String msg = "Cannot remove the everyone role.";
            log.info(msg);
            throw new RegistryException(msg);
        }

        try {
            userStoreAdmin.deleteRole(roleName);
        } catch (UserManagerException e) {
            String msg = "Could not delete the role. \nCaused by " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }
    }

    public String[] getAllUsers() throws RegistryException {

        try {
            return userStoreReader.getAllUserNames();
        } catch (UserManagerException e) {
            String msg = "Could not get all users of the Registry. Caused by: " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }
    }

    public String[] getAllRoles() throws RegistryException {

        try {
            return userStoreReader.getAllRoleNames();
        } catch (UserManagerException e) {
            String msg = "Could not get all roles of the Registry. Caused by: " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }
    }

    public boolean userExists(String userName) throws RegistryException {

        String[] allUsers = getAllUsers();

        for (int i = 0; i < allUsers.length; i++) {
            if (userName.equals(allUsers[i])) {
                return true;
            }
        }

        return false;
    }

    public Map getUserProperties(String userName) throws RegistryException {
        Map properties;
        try {
            properties = userStoreReader.getUserProperties(userName);
        } catch (UserManagerException e) {
            throw new RegistryException(e.getMessage());
        }
        return properties;
    }

    public void setUserProperties(String userName, Map properties) throws RegistryException {

        try {
            userStoreAdmin.setUserProperties(userName, properties);
        } catch (UserManagerException e) {
            throw new RegistryException(e.getMessage());
        }
    }

    public String getUserProperty(String userName, String propName) throws RegistryException {

        try {
            Map properties = userStoreReader.getUserProperties(userName);
            return (String)properties.get(propName);
        } catch (UserManagerException e) {
            throw new RegistryException(e.getMessage());
        }
    }

    public void setUserProperty(String userName, String name, String value)
            throws RegistryException {

        try {
            Map properties = userStoreReader.getUserProperties(userName);
            properties.put(name, value);
            userStoreAdmin.setUserProperties(userName, properties);

        } catch (UserManagerException e) {
            throw new RegistryException(e.getMessage());
        }
    }

    public void authorizeUser(String userName, String resourceID, String action)
            throws RegistryException {

        try {
            accessControlAdmin.authorizeUser(userName, resourceID, action);
        } catch (UserManagerException e) {
            String msg = "Could not authorize. \nCaused by " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }
    }

    public void authorizeRole(String roleName, String resourceID, String action)
            throws RegistryException {

        try {
            accessControlAdmin.authorizeRole(roleName, resourceID, action);
        } catch (UserManagerException e) {
            String msg = "Could not authorize. \nCaused by " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }
    }

    public void denyUser(String userName, String resourceID, String action)
            throws RegistryException {

        if (userName.equals(RegistryConstants.ADMIN_USER)) {
            String msg = "Cannot deny access for the administrator.";
            log.info(msg);
            throw new RegistryException(msg);
        }

        if (userName.equals(RegistryConstants.SYSTEM_USER)) {
            String msg = "Cannot deny access for the system user.";
            log.info(msg);
            throw new RegistryException(msg);
        }

        try {
            accessControlAdmin.denyUser(userName, resourceID, action);
        } catch (UserManagerException e) {
            String msg = "Could not deny. \nCaused by " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }
    }

    public void denyRole(String roleName, String resourceID, String action)
            throws RegistryException {

        if (roleName.equals(RegistryConstants.ADMIN_ROLE)) {
            String msg = "Cannot deny access for the administrator role.";
            log.info(msg);
            throw new RegistryException(msg);
        }

        try {
            accessControlAdmin.denyRole(roleName, resourceID, action);
        } catch (UserManagerException e) {
            String msg = "Could not deny. \nCaused by " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }
    }

    public boolean isUserAuthorized(String userName, String resourcePath, String action)
            throws RegistryException {

        try {
            return authorizer.isUserAuthorized(userName, resourcePath, action);
        } catch (UserManagerException e) {
            String msg = "Could not get the permissions for the user " +
                         userName + " for the resource " + resourcePath;
            log.error(msg, e);
            throw new RegistryException(msg);
        }
    }

    public void addUserToRole(String userName, String roleName) throws RegistryException {

        //try {
        //    if (!authorizer.isUserAuthorized(
        //            actor, RegistryConstants.ROOT_PATH, ActionConstants.AUTHORIZE)) {
        //        String msg = "Attempted to perform unauthorized operation.";
        //        log.info(msg);
        //        throw new RegistryException(msg);
        //    }
        //} catch (UserManagerException e) {
        //    String msg = "Could not check authorization. \nCaused by " + e.getMessage();
        //    log.error(msg, e);
        //    throw new RegistryException(msg);
        //}

        try {
            userStoreAdmin.addUserToRole(userName, roleName);
        } catch (UserManagerException e) {
            String msg = "Could not add user " + userName + " to role " + roleName + ".";
            log.error(msg, e);
            throw new RegistryException(msg);
        }
    }

    public void removeUserFromRole(String userName, String roleName) throws RegistryException {

        //try {
        //    if (!authorizer.isUserAuthorized(
        //            actor, RegistryConstants.ROOT_PATH, ActionConstants.AUTHORIZE)) {
        //        String msg = "Attempted to perform unauthorized operation.";
        //        log.info(msg);
        //        throw new RegistryException(msg);
        //    }
        //} catch (UserManagerException e) {
        //    String msg = "Could not check authorization. \nCaused by " + e.getMessage();
        //    log.error(msg, e);
        //    throw new RegistryException(msg);
        //}

        try {
            userStoreAdmin.removeUserFromRole(userName, roleName);
        } catch (UserManagerException e) {
            String msg = "Could not remove user " + userName + " from role " + roleName + ".";
            log.error(msg, e);
            throw new RegistryException(msg);
        }
    }
}
