/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */

package org.wso2.registry.inmemory;

import junit.framework.TestCase;
import org.wso2.registry.*;

import java.util.Properties;

public class InMemoryRegistryTest extends TestCase {

    /**
     * Registry instance for use in tests. Note that there should be only one Registry instance in a
     * JVM.
     */
    private static Registry registry = null;

    public void setUp() {
        registry = new InMemoryRegistry();
    }

    public void testFlatResourceHandling() {

        Resource r1 = new Resource();
        r1.setAuthorUserName("Chathura");
        r1.setDescription("This is a test resource used for registry testing.");
        String r1Content = "<c>This is r1 content</c>";
        r1.setContent(r1Content.getBytes());

        try {
            registry.put("/r1", r1);
        } catch (RegistryException e) {
            fail("Couldn't put content resource /r1");
        }

        Resource r1f = null;
        try {
            r1f = registry.get("/r1");
        } catch (RegistryException e) {
            fail("Couldn't get content resource /r1");
        }

        assertEquals("Content is not equal.",
                     new String((byte[])r1.getContent()), new String((byte[])r1f.getContent()));

        assertEquals("Author is not equal.", r1.getAuthorUserName(), r1f.getAuthorUserName());

        assertEquals("Description is not equal.", r1.getDescription(), r1f.getDescription());

        try {
            registry.delete("/r1");
        } catch (RegistryException e) {
            fail("Couldn't delete content resource /r1");
        }

        boolean failed = false;
        try {
            registry.get("/r1");
        } catch (RegistryException e) {
            failed = true;
        }
        assertTrue("Deleted resource /r1 still exist.", failed);
    }

    public void testHierarchicalResourceHandling() {

        // add a resource

        Resource r1 = new Resource();
        r1.setAuthorUserName("Author R1");
        byte[] r1content = "R1 content".getBytes();
        r1.setContent(r1content);

        try {
            registry.put("/d1/r1", r1);
        } catch (RegistryException e) {
            fail("Valid put resource scenario failed.");
        }

        Resource d1 = null;
        try {
            d1 = registry.get("/d1");
        } catch (RegistryException e) {
            fail("parent directory was not cerated.");
        }

        assertTrue("/d1 should be a collection.", d1.isDirectory());

        assertTrue("Content of /d1 should be a String[]", d1.getContent() instanceof String[]);

        String[] children = (String[])d1.getContent();
        boolean found = false;
        for (int i = 0; i < children.length; i++) {
            if (children[i].equals("/d1/r1")) {
                found = true;
                break;
            }
        }
        assertTrue("/d1/r1 should be a child of /d1", found);

        Resource r1f = null;
        try {
            r1f = registry.get("/d1/r1");
        } catch (RegistryException e) {
            fail("resource was not stored.");
        }

        assertEquals("Resource content is not stored correctly.",
                     new String((byte[])r1.getContent()), new String((byte[])r1f.getContent()));

        try {
            registry.delete("/d1");
        } catch (RegistryException e) {
            fail("Valid resource delete scenario failed.");
        }

        boolean failed1 = false;
        try {
            registry.get("/d1");
        } catch (RegistryException e) {
            failed1 = true;
        }
        assertTrue("Deleted collection /d1 still exist.", failed1);

        boolean failed2 = false;
        try {
            registry.get("/d1/r1");
        } catch (RegistryException e) {
            failed2 = true;
        }
        assertTrue("Child /d1/r1 of the deleted collection /d1 still exist.", failed2);
    }

    public void testTagging() {

        // add a resource

        try {
            Resource r1 = new Resource();
            r1.setAuthorUserName("Author R1");
            byte[] r1content = "R1 content".getBytes();
            r1.setContent(r1content);
            registry.put("/d1/r1", r1);

        } catch (RegistryException e) {
            fail("Failed to put test resources.");
        }

        try {
            registry.applyTag("/d1/r1", "jsp");
        } catch (RegistryException e) {
            fail("Valid tagging scenario failed");
        }

        TaggedResourcePath[] paths = null;
        try {
            paths = registry.getResourcePathsWithTag("jsp");
        } catch (RegistryException e) {
            fail("Failed to get resources with tag 'jsp'");
        }
        boolean artifactFound = false;
        for (int i = 0; i < paths.length; i++) {
            if (paths[i].getResourcePath().equals("/d1/r1")) {
                artifactFound = true;
                break;
            }
        }
        assertTrue("/d1/r1 is not tagged with the tag \"jsp\"", artifactFound);

        Tag[] tags = null;
        try {
            tags = registry.getTags("/d1/r1");
        } catch (RegistryException e) {
            fail("Failed to get tags for the resource /d1/r1");
        }

        boolean tagFound = false;
        for (int i = 0; i < tags.length; i++) {
            if (tags[i].getTagName().equals("jsp")) {
                tagFound = true;
                break;
            }
        }
        assertTrue("tag 'jsp' is not associated with the artifact /d1/r1", tagFound);

        try {
            registry.delete("/d1");
        } catch (RegistryException e) {
            fail("Failed to delete test resources.");
        }

        TaggedResourcePath[] paths2 = null;
        try {
            paths2 = registry.getResourcePathsWithTag("jsp");
        } catch (RegistryException e) {
            fail("Failed to get resources with tag 'jsp'");
        }
        boolean artifactNotFound = true;
        for (int i = 0; i < paths2.length; i++) {
            if (paths2[i].getResourcePath().equals("/d1/r1")) {
                artifactNotFound = false;
                break;
            }
        }
        assertTrue("Tagging 'jsp' still exist for the deleted resource /d1/r1", artifactNotFound);
    }

    public void testComments() {

        // add a resource

        try {
            Resource r1 = new Resource();
            r1.setAuthorUserName("Author R1");
            byte[] r1content = "R1 content".getBytes();
            r1.setContent(r1content);
            registry.put("/d1/r1", r1);

        } catch (RegistryException e) {
            fail("Failed to put test resources.");
        }

        String comment1 = "this can be used as a test resource.";
        try {
            registry.addComment("/d1/r1", new Comment(comment1));
        } catch (RegistryException e) {
            fail("Valid tagging scenario failed");
        }

        Comment[] comments = null;
        try {
            comments = registry.getComments("/d1/r1");
        } catch (RegistryException e) {
            fail("Failed to get comments for the resource /d1/r1");
        }

        boolean commentFound = false;
        for (int i = 0; i < comments.length; i++) {
            if (comments[i].getText().equals(comment1)) {
                commentFound = true;
                break;
            }
        }
        assertTrue(
                "comment '" + comment1 + "' is not associated with the artifact /d1/r1",
                commentFound);

        try {
            registry.delete("/d1");
        } catch (RegistryException e) {
            fail("Failed to delete test resources.");
        }
    }

    public void testRatings() {

        // add a resource

        try {
            Resource r1 = new Resource();
            r1.setAuthorUserName("Author R1");
            byte[] r1content = "R1 content".getBytes();
            r1.setContent(r1content);
            registry.put("/d1/r1", r1);

        } catch (RegistryException e) {
            fail("Failed to put test resources.");
        }

        try {
            registry.rateResource("/d1/r1", 4);
        } catch (RegistryException e) {
            fail("Couldn't rate the resource /d1/r1");
        }

        float rating = 0;
        try {
            rating = registry.getAverageRating("/d1/r1");
        } catch (RegistryException e) {
            fail("Couldn't get the rating of the resource /d1/r1");
        }

        assertEquals("Rating of the resource /d1/r1 should be 4.", rating, (float)4.0, (float)0.01);
        try {
            registry.delete("/d1");
        } catch (RegistryException e) {
            fail("Failed to delete test resources.");
        }
    }

    public void testCombinedScenario() throws RegistryException {

        // put a content resource in to the root

        String r1Content = "this is the r1 content.";
        Resource r1 = new Resource();
        r1.setContent(r1Content.getBytes());

        try {
            registry.put("/r1", r1);
        } catch (RegistryException e) {
            fail("Couldn't put a content resource in to path /r1");
        }

        // put a collection in to the root

        Resource c1 = new Resource();
        c1.setDirectory(true);

        try {
            registry.put("/c1", c1);
        } catch (RegistryException e) {
            fail("Couldn't put a collection artifact in to path /c1");
        }

        // put a content artifact in to /c1/r2

        String r2Content = "this is r2 content";
        Resource r2 = new Resource();
        r2.setContent(r2Content.getBytes());

        try {
            registry.put("/c1/r2", r2);
        } catch (RegistryException e) {
            fail("Couldn't put new content resource in to existing collection /c1");
        }

        // put a content artifact in to non-existing collection

        String r3Content = "this is r3 content";
        Resource r3 = new Resource();
        Properties props = new Properties();
        props.put("Reviewer", "Deepal");
        props.put("Tested on", "Axis2");
        r3.setProperties(props);
        r3.setContent(r3Content.getBytes());

        try {
            registry.put("/c2/r3", r3);
        } catch (RegistryException e) {
            fail("Couldn't put new content resource in to non-existing collection /c2");
        }

        // put c2/r4
        String r4Content = "this is r4 content";
        Resource r4 = new Resource();
        r4.setContent(r4Content.getBytes());

        registry.put("/c2/r4", r4);

        // do some taggings

        try {
            registry.applyTag("/c2/r4", "ui");
            registry.applyTag("/c2/r4", "swing");
            registry.applyTag("/r1", "test");
            registry.applyTag("/r1", "Java");
            registry.applyTag("/c1", "test");
            registry.applyTag("/c1/r2", "struts");
            registry.applyTag("/c1/r2", "ui");
            registry.applyTag("/c1/r2", "web");
            registry.applyTag("/c1/r2", "web2");
            registry.applyTag("/c1/r2", "web3");

        } catch (Exception e) {
            fail("Tagging artifacts failed.");
        }

        // add some comments

        try {
            registry.addComment("/r1", new Comment("this is a test artifact."));
            registry.addComment("/c1", new Comment("I don't agree with having this collection."));
            registry.addComment("/c1/r2", new Comment("Is this a collection ;)"));
            registry.addComment("/c2",
                                new Comment("I can understand the purpose of this. Thanks."));
        } catch (RegistryException e) {
            fail("Valid commenting scenario failed.");
        }

        // rate some artifacts

        try {
            registry.rateResource("/r1", 3);
            registry.rateResource("/r1", 4);
            registry.rateResource("/r1", 5);
            registry.rateResource("/c1", 2);
            registry.rateResource("/c1/r2", 5);
            registry.rateResource("/c2/r4", 1);
            registry.rateResource("/c2/r4", 2);

        } catch (RegistryException e) {
            fail("Valid rating scenario failed.");
        }

        registry.delete("/r1");
        registry.delete("/c1");
        registry.delete("/c2");
    }
}
