/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc;

import junit.framework.TestCase;
import org.wso2.registry.*;
import org.wso2.registry.jdbc.realm.InMemoryRegistryRealm;
import org.wso2.registry.jdbc.realm.RegistryRealm;
import org.wso2.registry.secure.SecureRegistry;
import org.wso2.usermanager.Realm;
import org.wso2.usermanager.UserManagerException;

import java.text.DateFormat;
import java.text.ParseException;
import java.util.Date;

public class TagsTest extends TestCase {

    /**
     * Registry instance for use in tests. Note that there should be only one Registry instance in a
     * JVM.
     */
    protected static Registry registry = null;
    protected static RegistryRealm realm = null;

    public void setUp() {
        try {
            if (registry == null) {
                realm = new InMemoryRegistryRealm();
                registry = new InMemoryJDBCRegistry(realm);
            }
        } catch (RegistryException e) {
            fail("Failed to initialize the registry.");
        }
    }

    public void testFormat() {

        try {
            DateFormat format = DateFormat.getDateInstance(DateFormat.SHORT);

            Date date = format.parse("2007/1/22");
            int i = 1;
        } catch (ParseException e) {
            e.printStackTrace();
        }

    }

    public void testTagging() {
        // add a resource

        try {
            Resource r1 = new Resource();
            r1.setAuthorUserName("Author R1");
            byte[] r1content = "R1 content".getBytes();
            r1.setContent(r1content);
            registry.put("/d11/r1", r1);

            Resource r2 = new Resource();
            r2.setAuthorUserName("Author R2");
            byte[] r2content = "R2 content".getBytes();
            r2.setContent(r2content);
            registry.put("/d11/r2", r2);

            Resource r3 = new Resource();
            r3.setAuthorUserName("Author R3");
            byte[] r3content = "R3 content".getBytes();
            r3.setContent(r3content);
            registry.put("/d11/r3", r3);

        } catch (RegistryException e) {
            fail("Failed to put test resources.");
        }

        try {
            registry.applyTag("/d11/r1", "jsp");
            registry.applyTag("/d11/r2", "jsp");
            registry.applyTag("/d11/r3", "java");
        } catch (RegistryException e) {
            fail("Valid tagging scenario failed");
        }

        TaggedResourcePath[] paths = null;
        try {
            paths = registry.getResourcePathsWithTag("jsp");
        } catch (RegistryException e) {
            fail("Failed to get resources with tag 'jsp'");
        }
        boolean artifactFound = false;
        for (int i = 0; i < paths.length; i++) {
            if (paths[i].getResourcePath().equals("/d11/r1")) {
                artifactFound = true;
                break;
            }
        }
        assertTrue("/d11/r1 is not tagged with the tag \"jsp\"", artifactFound);

        Tag[] tags = null;
        try {
            tags = registry.getTags("/d11/r1");
        } catch (RegistryException e) {
            fail("Failed to get tags for the resource /d11/r1");
        }

        boolean tagFound = false;
        for (int i = 0; i < tags.length; i++) {
            if (tags[i].getTagName().equals("jsp")) {
                tagFound = true;
                break;
            }
        }
        assertTrue("tag 'jsp' is not associated with the artifact /d11/r1", tagFound);

        try {
            registry.delete("/d11");
        } catch (RegistryException e) {
            fail("Failed to delete test resources.");
        }

        TaggedResourcePath[] paths2 = null;
        try {
            paths2 = registry.getResourcePathsWithTag("jsp");
        } catch (RegistryException e) {
            fail("Failed to get resources with tag 'jsp'");
        }

        assertEquals("Tag based search should not return paths of deleted resources.",
                     paths2.length, 0);
    }

    public void testMultipleUserTags() {

        try {
            SecureRegistry adminRegistry =
                    new SecureRegistry(RegistryConstants.ADMIN_USER, "admin", registry, realm);
            Realm adminRealm = adminRegistry.getUserRealm();

            String r1Content = "R1";
            Resource r1 = new Resource();
            r1.setContent(r1Content.getBytes());

            try {
                adminRegistry.put("/r1", r1);
            } catch (RegistryException e) {
                fail("Valid put scenario failed.");
            }

            try {
                adminRealm.getUserStoreAdmin().addUser("chathura", "cce");
                adminRealm.getUserStoreAdmin().addUser("samitha", "swe");
            } catch (UserManagerException e) {
                e.printStackTrace();
            }

            SecureRegistry chathuraRegistry =
                    new SecureRegistry("chathura", "cce", registry, realm);
            SecureRegistry samithaRegistry = new SecureRegistry("samitha", "swe", registry, realm);

            adminRegistry.applyTag("/r1", "java");
            chathuraRegistry.applyTag("/r1", "java");
            samithaRegistry.applyTag("/r1", "java");

            Tag[] tags1 = adminRegistry.getTags("/r1");
            assertEquals("There should be 3 taggings on resource '/r1'", 3, tags1[0].getTagCount());

            chathuraRegistry.removeTag("/r1", "java");
            Tag[] tags2 = adminRegistry.getTags("/r1");
            assertEquals("There should be 2 taggings on resource '/r1'", 2, tags2[0].getTagCount());

            adminRegistry.removeTag("/r1", "java");
            Tag[] tags3 = adminRegistry.getTags("/r1");
            assertEquals("There should not be any taggings on resource '/r1'", 0, tags3.length);

        } catch (RegistryException e) {
            e.printStackTrace();
        }
    }
}
