/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.actions;

import org.wso2.registry.*;
import org.wso2.registry.servlet.utils.UserUtil;
import org.wso2.registry.secure.SecureRegistry;
import org.wso2.registry.web.actions.utils.Permission;
import org.wso2.registry.web.actions.utils.ResourcePath;
import org.wso2.registry.web.utils.CommonUtil;
import org.wso2.usermanager.Realm;
import org.wso2.usermanager.UserManagerException;
import org.wso2.usermanager.UserManagerConstants;

import javax.servlet.http.HttpServletRequest;
import java.util.*;

public class ResourceDetailsAction extends AbstractRegistryAction {

    private String path;

    /**
     * This class may contain details of the current version of resources or older versions of
     * resources. When browsing an older version of a resource, it may be necessary to get the
     * path of the current version of that resource. This field contains the path of the
     * current (active) version of the resource. If this object contains the details of the
     * current version, path feild and the activeResourcePath field will be identical. 
     */
    private String activeResourcePath;

    private String relativePath;
    private String contentPath;
    private String permalink;
    private String name;
    private String author;
    private String lastUpdater;
    private String description;
    private Date createdOn;
    private Date lastModified;
    private String mediaType;
    private boolean collection;
    private boolean putAllowed;
    private boolean deleteAllowed;
    private boolean authorizeAllowed;
    private Properties properties;
    private boolean versionView;

    private List tags = new ArrayList();
    private List comments = new ArrayList();
    private float averageRating;
    private List userPermissions = new ArrayList();
    private List rolePermissions = new ArrayList();

    private List navigatablePaths = new ArrayList();

    private String parentPath;

    /**
     * Used to populate the users in the userPermissions section
     */
    private List userNames = new ArrayList();

    /**
     * Used to populate the roles in the rolePermissions section
     */
    private List roleNames = new ArrayList();

    public String execute(HttpServletRequest request) throws RegistryException {

        setRequest(request);

        Registry registry = getRegistry();

        // if path is not set, we assume that root "/" is requested
        if (path == null || path.length() == 0) {
            path = RegistryConstants.ROOT_PATH;
        }

        if (path.equals(RegistryConstants.ROOT_PATH)) {

            ResourcePath resourcePath = new ResourcePath();
            resourcePath.setNavigateName("/");
            resourcePath.setNavigatePath("#");

            navigatablePaths.add(resourcePath);

        } else {

            // first add the root path
            ResourcePath rootPath = new ResourcePath();
            rootPath.setNavigateName("/");
            rootPath.setNavigatePath(RegistryConstants.ROOT_PATH);
            navigatablePaths.add(rootPath);

            String preparedPath = path;
            if (preparedPath.startsWith(RegistryConstants.PATH_SEPARATOR)) {
                preparedPath = preparedPath.substring(RegistryConstants.PATH_SEPARATOR.length());
            }

            if (preparedPath.endsWith(RegistryConstants.PATH_SEPARATOR)) {
                preparedPath = preparedPath.
                        substring(0, preparedPath.length() - RegistryConstants.PATH_SEPARATOR.length());
            }

            String[] parts = preparedPath.split(RegistryConstants.PATH_SEPARATOR);
            for (int i = 0; i < parts.length; i++) {
                ResourcePath resourcePath = new ResourcePath();

                if (i == parts.length - 1) {
                    String[] q = parts[i].split("\\?");
                    if (q.length == 2) {
                        if (q[1].startsWith("v=")) {
                            String versionNumber = q[1].substring("v=".length());
                            String navName = q[0] + " (version " + versionNumber + ")";
                            resourcePath.setNavigateName(navName);
                            this.versionView = true;
                        } else {
                            resourcePath.setNavigateName(parts[i]);
                        }
                    } else {
                        resourcePath.setNavigateName(parts[i]);
                    }
                } else {
                    resourcePath.setNavigateName(parts[i]);
                }

                String tempPath = "";
                for (int j = 0; j < i + 1; j++) {
                    tempPath = tempPath + RegistryConstants.PATH_SEPARATOR + parts[j];
                }
                resourcePath.setNavigatePath(tempPath);
                navigatablePaths.add(resourcePath);
            }
        }

        if (RegistryConstants.ROOT_PATH.equals(path)) {
            relativePath = "";
        } else {
            if (path.startsWith(RegistryConstants.PATH_SEPARATOR)) {
                relativePath = path.substring(1, path.length());
            }
        }

        if (path.equals(RegistryConstants.ROOT_PATH)) {
            name = "root";
        } else {
            String[] parts = path.split(RegistryConstants.PATH_SEPARATOR);
            name = parts[parts.length - 1];

            String[] q = name.split("\\?");
            if (q.length == 2) {
                if (q[1].startsWith("v=")) {
                    String versionNumber = q[1].substring("v=".length());
                    name = q[0] + " (version " + versionNumber + ")";
                }
            }
        }

        contentPath = path;
        //String[] qParts = path.split("\\?");
        //if (qParts.length > 1) {
        //    contentPath = path + "&content";
        //} else {
        //    contentPath = path + "?content";
        //}

        Resource resource = registry.get(path);

        if (!versionView) {
            String[] versionPaths = registry.getVersions(path);
            if (versionPaths.length > 0) {
                permalink = versionPaths[0];
            }
        } else {
            permalink = path;
        }

        if (versionView) {

            String[] parts = path.split("\\?v=");
            if (parts.length == 2) {
                activeResourcePath = parts[0];
            } else {
                activeResourcePath = path;
            }
            
        } else {
            activeResourcePath = path;
        }

        if (resource.getAuthorUserName() != null) {
            this.author = resource.getAuthorUserName();
        }

        if (resource.getDescription() != null) {
            this.description = resource.getDescription();
        }

        this.collection = resource.isDirectory();

        this.path = resource.getPath();
        this.createdOn = resource.getCreatedTime();
        this.lastModified = resource.getLastModified();
        this.lastUpdater = resource.getLastUpdaterUserName();
        this.mediaType = resource.getMediaType();
        this.averageRating = registry.getAverageRating(resource.getPath());
        this.properties = resource.getProperties();

        Tag[] t = registry.getTags(resource.getPath());
        for (int i = 0; i < t.length; i++) {
            tags.add(t[i]);
        }

        Comment[] c = registry.getComments(resource.getPath());
        for (int i = 0; i < c.length; i++) {
            comments.add(c[i]);
        }

        // construct the parent path
        if (path.equals(RegistryConstants.ROOT_PATH)) {
            parentPath = "root";
        } else {
            String formattedPath = path;
            if (path.endsWith(RegistryConstants.PATH_SEPARATOR)) {
                formattedPath = path.substring(
                        0, path.length() - RegistryConstants.PATH_SEPARATOR.length());
            }

            if (formattedPath.lastIndexOf(RegistryConstants.PATH_SEPARATOR) == 0) {
                parentPath = RegistryConstants.ROOT_PATH;
            } else {
                parentPath = formattedPath.substring(
                        0, formattedPath.lastIndexOf(RegistryConstants.PATH_SEPARATOR));
            }
        }

        try {

            Realm userRealm = ((SecureRegistry) registry).getUserRealm();
            String[] userArray = userRealm.getUserStoreReader().getAllUserNames();
            for (int i = 0; i < userArray.length; i++) {
                userNames.add(userArray[i]);
            }

            String[] rolesArray = userRealm.getUserStoreReader().getAllRoleNames();
            for (int i = 0; i < rolesArray.length; i++) {
                roleNames.add(rolesArray[i]);
            }

            String authorizationPath = path;
            if (path.indexOf("?") > 0) {
                authorizationPath = path.split("\\?")[0];
            } else if (path.indexOf(";")>0) {
                authorizationPath = path.split("\\;")[0];
            }

            if (UserUtil.isPutAllowed(getUserName(), authorizationPath, request)) {
                putAllowed = true;
            }

            if (UserUtil.isDeleteAllowed(getUserName(), authorizationPath, request)) {
                deleteAllowed = true;
            }

            if (UserUtil.isAuthorizeAllowed(getUserName(), authorizationPath, request)) {
                authorizeAllowed = true;
            }

            Map userPermissionMap = new HashMap();

            String[] raUsers = userRealm.getAuthorizer().getAllowedUsersForResource(path, ActionConstants.GET);
            for (int i = 0; i < raUsers.length; i++) {
                if (userPermissionMap.containsKey(raUsers[i])) {
                    Permission permission = (Permission) userPermissionMap.get(raUsers[i]);
                    permission.setReadAllow(true);
                } else {
                    Permission permission = new Permission();
                    permission.setUserName(raUsers[i]);
                    permission.setReadAllow(true);
                    userPermissionMap.put(raUsers[i], permission);
                }
            }

            String[] rdUsers = userRealm.getAuthorizer().getDeniedUsersForResource(path, ActionConstants.GET);
            for (int i = 0; i < rdUsers.length; i++) {
                if (userPermissionMap.containsKey(rdUsers[i])) {
                    Permission permission = (Permission) userPermissionMap.get(rdUsers[i]);
                    permission.setReadDeny(true);
                } else {
                    Permission permission = new Permission();
                    permission.setUserName(rdUsers[i]);
                    permission.setReadDeny(true);
                    userPermissionMap.put(rdUsers[i], permission);
                }
            }

            String[] waUsers = userRealm.getAuthorizer().getAllowedUsersForResource(path, ActionConstants.PUT);
            for (int i = 0; i < waUsers.length; i++) {
                if (userPermissionMap.containsKey(waUsers[i])) {
                    Permission permission = (Permission) userPermissionMap.get(waUsers[i]);
                    permission.setWriteAllow(true);
                } else {
                    Permission permission = new Permission();
                    permission.setUserName(waUsers[i]);
                    permission.setWriteAllow(true);
                    userPermissionMap.put(waUsers[i], permission);
                }
            }

            String[] wdUsers = userRealm.getAuthorizer().getDeniedUsersForResource(path, ActionConstants.PUT);
            for (int i = 0; i < wdUsers.length; i++) {
                if (userPermissionMap.containsKey(wdUsers[i])) {
                    Permission permission = (Permission) userPermissionMap.get(wdUsers[i]);
                    permission.setWriteDeny(true);
                } else {
                    Permission permission = new Permission();
                    permission.setUserName(wdUsers[i]);
                    permission.setWriteDeny(true);
                    userPermissionMap.put(wdUsers[i], permission);
                }
            }

            String[] daUsers = userRealm.getAuthorizer().getAllowedUsersForResource(path, ActionConstants.DELETE);
            for (int i = 0; i < daUsers.length; i++) {
                if (userPermissionMap.containsKey(daUsers[i])) {
                    Permission permission = (Permission) userPermissionMap.get(daUsers[i]);
                    permission.setDeleteAllow(true);
                } else {
                    Permission permission = new Permission();
                    permission.setUserName(daUsers[i]);
                    permission.setDeleteAllow(true);
                    userPermissionMap.put(daUsers[i], permission);
                }
            }

            String[] ddUsers = userRealm.getAuthorizer().getDeniedUsersForResource(path, ActionConstants.DELETE);
            for (int i = 0; i < ddUsers.length; i++) {
                if (userPermissionMap.containsKey(ddUsers[i])) {
                    Permission permission = (Permission) userPermissionMap.get(ddUsers[i]);
                    permission.setDeleteDeny(true);
                } else {
                    Permission permission = new Permission();
                    permission.setUserName(ddUsers[i]);
                    permission.setDeleteDeny(true);
                    userPermissionMap.put(ddUsers[i], permission);
                }
            }

            String[] aaUsers = userRealm.getAuthorizer().getAllowedUsersForResource(path, UserManagerConstants.AUTHORIZE);
            for (int i = 0; i < aaUsers.length; i++) {
                if (userPermissionMap.containsKey(aaUsers[i])) {
                    Permission permission = (Permission) userPermissionMap.get(aaUsers[i]);
                    permission.setAuthorizeAllow(true);
                } else {
                    Permission permission = new Permission();
                    permission.setUserName(aaUsers[i]);
                    permission.setAuthorizeAllow(true);
                    userPermissionMap.put(aaUsers[i], permission);
                }
            }

            String[] adUsers = userRealm.getAuthorizer().getDeniedUsersForResource(path, UserManagerConstants.AUTHORIZE);
            for (int i = 0; i < adUsers.length; i++) {
                if (userPermissionMap.containsKey(adUsers[i])) {
                    Permission permission = (Permission) userPermissionMap.get(adUsers[i]);
                    permission.setAuthorizeDeny(true);
                } else {
                    Permission permission = new Permission();
                    permission.setUserName(adUsers[i]);
                    permission.setAuthorizeDeny(true);
                    userPermissionMap.put(adUsers[i], permission);
                }
            }

            userPermissions = new ArrayList(userPermissionMap.values());

            Map rolePermissionMap = new HashMap();

            String[] raRoles = userRealm.getAuthorizer().getAllowedRolesForResource(path, ActionConstants.GET);
            for (int i = 0; i < raRoles.length; i++) {
                if (rolePermissionMap.containsKey(raRoles[i])) {
                    Permission permission = (Permission) rolePermissionMap.get(raRoles[i]);
                    permission.setReadAllow(true);
                } else {
                    Permission permission = new Permission();
                    permission.setUserName(raRoles[i]);
                    permission.setReadAllow(true);
                    rolePermissionMap.put(raRoles[i], permission);
                }
            }

            String[] rdRoles = userRealm.getAuthorizer().getDeniedRolesForResource(path, ActionConstants.GET);
            for (int i = 0; i < rdRoles.length; i++) {
                if (rolePermissionMap.containsKey(rdRoles[i])) {
                    Permission permission = (Permission) rolePermissionMap.get(rdRoles[i]);
                    permission.setReadDeny(true);
                } else {
                    Permission permission = new Permission();
                    permission.setUserName(rdRoles[i]);
                    permission.setReadDeny(true);
                    rolePermissionMap.put(rdRoles[i], permission);
                }
            }

            String[] waRoles = userRealm.getAuthorizer().getAllowedRolesForResource(path, ActionConstants.PUT);
            for (int i = 0; i < waRoles.length; i++) {
                if (rolePermissionMap.containsKey(waRoles[i])) {
                    Permission permission = (Permission) rolePermissionMap.get(waRoles[i]);
                    permission.setWriteAllow(true);
                } else {
                    Permission permission = new Permission();
                    permission.setUserName(waRoles[i]);
                    permission.setWriteAllow(true);
                    rolePermissionMap.put(waRoles[i], permission);
                }
            }

            String[] wdRoles = userRealm.getAuthorizer().getDeniedRolesForResource(path, ActionConstants.PUT);
            for (int i = 0; i < wdRoles.length; i++) {
                if (rolePermissionMap.containsKey(wdRoles[i])) {
                    Permission permission = (Permission) rolePermissionMap.get(wdRoles[i]);
                    permission.setWriteDeny(true);
                } else {
                    Permission permission = new Permission();
                    permission.setUserName(wdRoles[i]);
                    permission.setWriteDeny(true);
                    rolePermissionMap.put(wdRoles[i], permission);
                }
            }

            String[] daRoles = userRealm.getAuthorizer().getAllowedRolesForResource(path, ActionConstants.DELETE);
            for (int i = 0; i < daRoles.length; i++) {
                if (rolePermissionMap.containsKey(daRoles[i])) {
                    Permission permission = (Permission) rolePermissionMap.get(daRoles[i]);
                    permission.setDeleteAllow(true);
                } else {
                    Permission permission = new Permission();
                    permission.setUserName(daRoles[i]);
                    permission.setDeleteAllow(true);
                    rolePermissionMap.put(daRoles[i], permission);
                }
            }

            String[] ddRoles = userRealm.getAuthorizer().getDeniedRolesForResource(path, ActionConstants.DELETE);
            for (int i = 0; i < ddRoles.length; i++) {
                if (rolePermissionMap.containsKey(ddRoles[i])) {
                    Permission permission = (Permission) rolePermissionMap.get(ddRoles[i]);
                    permission.setDeleteDeny(true);
                } else {
                    Permission permission = new Permission();
                    permission.setUserName(ddRoles[i]);
                    permission.setDeleteDeny(true);
                    rolePermissionMap.put(ddRoles[i], permission);
                }
            }

            String[] aaRoles = userRealm.getAuthorizer().getAllowedRolesForResource(path, UserManagerConstants.AUTHORIZE);
            for (int i = 0; i < aaRoles.length; i++) {
                if (rolePermissionMap.containsKey(aaRoles[i])) {
                    Permission permission = (Permission) rolePermissionMap.get(aaRoles[i]);
                    permission.setAuthorizeAllow(true);
                } else {
                    Permission permission = new Permission();
                    permission.setUserName(aaRoles[i]);
                    permission.setAuthorizeAllow(true);
                    rolePermissionMap.put(aaRoles[i], permission);
                }
            }

            String[] adRoles = userRealm.getAuthorizer().getDeniedRolesForResource(path, UserManagerConstants.AUTHORIZE);
            for (int i = 0; i < adRoles.length; i++) {
                if (rolePermissionMap.containsKey(adRoles[i])) {
                    Permission permission = (Permission) rolePermissionMap.get(adRoles[i]);
                    permission.setAuthorizeDeny(true);
                } else {
                    Permission permission = new Permission();
                    permission.setUserName(adRoles[i]);
                    permission.setAuthorizeDeny(true);
                    rolePermissionMap.put(adRoles[i], permission);
                }
            }

            rolePermissions = new ArrayList(rolePermissionMap.values());

        } catch (UserManagerException e) {
            String msg = "Could not get user manager details. Caused by: " + e.getMessage();
            throw new RegistryException(msg);
        }

        return SUCCESS;
    }

    public String getPath() {
        return path;
    }

    public void setPath(String path) {
        this.path = path;
    }

    public String getActiveResourcePath() {
        return activeResourcePath;
    }

    public void setActiveResourcePath(String activeResourcePath) {
        this.activeResourcePath = activeResourcePath;
    }

    public String getRelativePath() {
        return relativePath;
    }

    public void setRelativePath(String relativePath) {
        this.relativePath = relativePath;
    }

    public String getContentPath() {
        return contentPath;
    }

    public void setContentPath(String contentPath) {
        this.contentPath = contentPath;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getAuthor() {
        return author;
    }

    public void setAuthor(String author) {
        this.author = author;
    }

    public String getLastUpdater() {
        return lastUpdater;
    }

    public void setLastUpdater(String lastUpdater) {
        this.lastUpdater = lastUpdater;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public Date getCreatedOn() {
        return createdOn;
    }

    public String getFormattedCreatedOn() {
        return CommonUtil.formatDate(createdOn);
    }

    public void setCreatedOn(Date createdOn) {
        this.createdOn = createdOn;
    }

    public Date getLastModified() {
        return lastModified;
    }

    public String getFormattedLastModified() {
        return CommonUtil.formatDate(lastModified);
    }

    public void setLastModified(Date lastModified) {
        this.lastModified = lastModified;
    }

    public String getMediaType() {
        return mediaType;
    }

    public void setMediaType(String mediaType) {
        this.mediaType = mediaType;
    }

    public boolean isCollection() {
        return collection;
    }

    public void setCollection(boolean collection) {
        this.collection = collection;
    }

    public boolean isPutAllowed() {
        return putAllowed;
    }

    public void setPutAllowed(boolean putAllowed) {
        this.putAllowed = putAllowed;
    }

    public boolean isDeleteAllowed() {
        return deleteAllowed;
    }

    public void setDeleteAllowed(boolean deleteAllowed) {
        this.deleteAllowed = deleteAllowed;
    }

    public boolean isAuthorizeAllowed() {
        return authorizeAllowed;
    }

    public void setAuthorizeAllowed(boolean authorizeAllowed) {
        this.authorizeAllowed = authorizeAllowed;
    }

    public Properties getProperties() {
        return properties;
    }

    public void setProperties(Properties properties) {
        this.properties = properties;
    }

    public float getAverageRating() {
        return averageRating;
    }

    public void setAverageRating(float averageRating) {
        this.averageRating = averageRating;
    }

    public List getTags() {
        return tags;
    }

    public void setTags(List tags) {
        this.tags = tags;
    }

    public List getComments() {
        return comments;
    }

    public void setComments(List comments) {
        this.comments = comments;
    }

    public String getParentPath() {
        return parentPath;
    }

    public void setParentPath(String parentPath) {
        this.parentPath = parentPath;
    }

    public List getUserNames() {
        return userNames;
    }

    public void setUserNames(List userNames) {
        this.userNames = userNames;
    }

    public List getRoleNames() {
        return roleNames;
    }

    public void setRoleNames(List roleNames) {
        this.roleNames = roleNames;
    }

    public List getUserPermissions() {
        return userPermissions;
    }

    public void setUserPermissions(List userPermissions) {
        this.userPermissions = userPermissions;
    }

    public List getRolePermissions() {
        return rolePermissions;
    }

    public void setRolePermissions(List rolePermissions) {
        this.rolePermissions = rolePermissions;
    }

    public List getNavigatablePaths() {
        return navigatablePaths;
    }

    public void setNavigatablePaths(List navigatablePaths) {
        this.navigatablePaths = navigatablePaths;
    }

    public boolean isVersionView() {
        return versionView;
    }

    public String getPermalink() {
        return permalink;
    }

    public void setPermalink(String permalink) {
        this.permalink = permalink;
    }
}
